import 'dart:convert';
import 'dart:io';
import 'package:http/http.dart' as http;
import 'package:image_picker/image_picker.dart';
import 'package:pea_cremation/domain/model/user.dart';
import 'package:pea_cremation/domain/service/base_service.dart';

class UserService extends BaseService {
  Future<User> login(
      String username, String password, int memberType, bool remember) async {
    final response = await http.post('$baseUrl/login',
        headers: requestHeaders,
        body: jsonEncode(<String, dynamic>{
          "username": username.length <13 ? username : "",
          "idcard": username.length == 13 ? username : "",
          "password": password,
          "member_type": '$memberType',
          "remember": remember
        }));

    if (response.statusCode == 200) {
      var result = json.decode(response.body)['results'];

      setToken(result['access_token']);

      User user = User.fromJson(Map<String, dynamic>.from(result['user']));
      saveUser(user);

      return user;
    } else {
      throw Exception(response.body);
    }
  }

  Future<User> idmLogin(
      String username, String password, int memberType, bool remember) async {
    print('$username $password $memberType $remember');

    final response = await http.post('$baseUrl/loginidm',
        headers: requestHeaders,
        body: jsonEncode(<String, dynamic>{
          "username": username,
          "password": password,
          "member_type": '$memberType',
          "remember": remember
        }));

    if (response.statusCode == 200) {
      var result = json.decode(response.body)['results'];

      setToken(result['access_token']);

      User user = User.fromJson(Map<String, dynamic>.from(result['user']));
      saveUser(user);

      return user;
    } else {
      throw Exception(response.body);
    }
  }

  Future<User> verifyToken() async {
    final response = await http.get('$baseUrl/validateToken',
        headers: await getAuthorizedHeader());

    if (response.statusCode == 200) {
      var result = json.decode(response.body)['results'];

      User user = User.fromJson(Map<String, dynamic>.from(result['user']));
      print('.....................');
      print('${user.toJson()}');
      print('.....................');
      saveUser(user);

      return user;
    } else {
      print('${await getAuthorizedHeader()}');
      print(response.body);
      throw Exception(response.body);
    }
  }

  Future<String> getUserToken(String memberCode, String clientID, String secret) async {
    clearUserData();

    final response = await http.post('$baseUrl/getToken',
        body: Map.of(<String, dynamic>{
          "client_id": clientID,
          "member_code": memberCode,
          "client_secret": secret
        }));

    print(response.body);

    if (response.statusCode == 200) {
      var result = json.decode(response.body)['results'];
      setToken(result['access_token']);

      return result['access_token'];
    } else {
      throw Exception(response.body);
    }
  }

  Future<User> logout() async {
    final response = await http.post('$baseUrl/logout',
        headers: await getAuthorizedHeader());

    if (response.statusCode == 200) {
      setToken(null);
      saveUser(null);

      return User.fromJson(json.decode(response.body));
    } else {
      throw Exception(response.body);
    }
  }

  Future<String> getPolicy() async {
    final response =
        await http.get('$baseUrl/getPolicy', headers: requestHeaders);

    if (response.statusCode == 200) {
      var result = jsonDecode(response.body)['results'];

      return Map<String, dynamic>.from(result[0])['messageDetail'];
    } else {
      throw Exception(response.body);
    }
  }

  Future<bool> applyPolicy() async {
    final response = await http.get('$baseUrl/applyPolicy',
        headers: await getAuthorizedHeader());

    return response.statusCode == 200;
  }

  void clearUserData() {
    setToken("");
  }

  Future<Map<String, dynamic>> preRegister(String memberCode, String firstName,
      String lastName, int memberType) async {
    final response = await http.get(
        '$baseUrl/preregister?member_code=$memberCode&fname=$firstName&lname=$lastName&member_type=$memberType',
        headers: requestHeaders);

    return jsonDecode(response.body);
  }

  Future<User> register(String memberCode, String idCard, String email,
      String password, int memberType) async {
    final response = await http.post('$baseUrl/register',
        headers: requestHeaders,
        body: jsonEncode(<String, dynamic>{
          "member_code": memberCode,
          "member_type": '$memberType',
          "idcard": idCard,
          "password": password,
          "email": email
        }));

    if (response.statusCode == 200) {
      var result = json.decode(response.body)['results'];

      setToken(result['access_token']);

      User user = User.fromJson(Map<String, dynamic>.from(result['user']));

      saveUser(user);

      return user;
    } else {
      return jsonDecode(response.body);
    }
  }

  Future<User> checkMember(
      String idCard, String email, int memberType) async {
    final response = await http.get(
        '$baseUrl/checkmember?idcard=$idCard&email=$email&member_type=$memberType',
        headers: requestHeaders);

    if (response.statusCode == 200) {
      var result = json.decode(response.body)['results'];

      setToken(result['access_token']);

      User user = User.fromJson(Map<String, dynamic>.from(result['user']));

      return user;
    } else {
      throw Exception(response.body);
    }
  }

  Future<Map<String, dynamic>> resetPassword(String memberCode, String password,
      String confirmPassword, int memberType) async {
    final response = await http.post('$baseUrl/resetpassword',
        headers: requestHeaders,
        body: jsonEncode(<String, dynamic>{
          "member_code": memberCode,
          "member_type": '$memberType',
          "password": password,
          "confirm_password": confirmPassword
        }));

    print(response.body);

    if (response.statusCode == 200) {
      var result = json.decode(response.body)['results'];

      // setToken(result['access_token']);

      // User user = User.fromJson(Map<String, dynamic>.from(result['user']));

      return result;
    } else {
      throw Exception(json.decode(response.body)['message']);
    }
  }

  Future<User> getProfile() async {
    final response = await http.get('$baseUrl/getProfile',
        headers: await getAuthorizedHeader());

    if (response.statusCode == 200) {
      var result = json.decode(response.body)['results'];
      print(result['user']);

      return User.fromJson(Map<String, dynamic>.of(result['user']));
    } else {
      throw Exception(response.body);
    }
  }

  Future<User> uploadPhoto(PickedFile file) async {
    final request =
        http.MultipartRequest('POST', Uri.parse('$baseUrl/uploadImage'));

    request.headers.addAll(await getAuthorizedHeader());
    request.files.add(await http.MultipartFile.fromPath("image", file.path));

    var response = await request.send();

    print('${response.statusCode}');

    if (response.statusCode == 200) {
      var result = json.decode(await response.stream.bytesToString())['results'];

    print('$result');

      return User.fromJson(Map<String, dynamic>.of(result['user']));
    } else {
      throw Exception(await response.stream.bytesToString());
    }
  }

  Future<User> uploadPhoto2(File file) async {
    final request =
        http.MultipartRequest('POST', Uri.parse('$baseUrl/uploadImage'));

    request.headers.addAll(await getAuthorizedHeader());
    request.files.add(await http.MultipartFile.fromPath("image", file.path));

    var response = await request.send();

      print('$response');

    if (response.statusCode == 200) {
      var result =
          json.decode(await response.stream.bytesToString())['results'];


      return User.fromJson(Map<String, dynamic>.of(result['user']));
    } else {
      throw Exception(await response.stream.bytesToString());
    }
  }

  Future<User> updateProfile(
      String email,
      String birthDay,
      String mobile,
      String address,
      String province,
      String amphure,
      String district,
      String zipCode,
      String receiveLetter) async {
    final response = await http.post('$baseUrl/updateProfile',
        headers: await getAuthorizedHeader(),
        body: jsonEncode(<String, dynamic>{
          "email": email,
          "birth_day": birthDay,
          "mobile": mobile,
          "address": address,
          "province_id": province == null ? null : province,
          "amphur_id": amphure == null ? null : amphure,
          "district_id": district == null ? null : district,
          "zipcode": zipCode,
          'isreceiveletter': receiveLetter
        }));

    if (response.statusCode == 200) {
      print(response.request.url);
      print('####################');
      print(jsonEncode(<String, dynamic>{
        "email": email,
        "birth_day": birthDay,
        "mobile": mobile,
        "address": address,
        "province_id": province == null ? null : province,
        "amphur_id": amphure == null ? null : amphure,
        "district_id": district == null ? null : district,
        "zipcode": zipCode,
        'isreceiveletter': receiveLetter
      }));
      print('####################');
      print(response.body);
      print('####################');

      var result = json.decode(response.body)['results'];

      User user = User.fromJson(Map<String, dynamic>.from(result['user']));

      saveUser(user);

      return user;
    } else {
      throw Exception(response.body);
    }
  }

  Future<Map<String, dynamic>> changePassword(String password, String confirmPassword) async {
    final response = await http.post('$baseUrl/resetpasswordbytoken',
        headers: await getAuthorizedHeader(),
        body: jsonEncode(<String, dynamic>{
          "password": password,
          "confirm_password": confirmPassword
        }));

    print(response.body);

    if (response.statusCode == 200) {
      var result = json.decode(response.body)['results'];

      // setToken(result['access_token']);

      // User user = User.fromJson(Map<String, dynamic>.from(result['user']));

      return result;
    } else {
      throw Exception(json.decode(response.body)['message']);
    }
  }
}
