import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:pea_cremation/pages/changepass/change_pass_view_model.dart';
import 'package:pea_cremation/shares/alert_dialog.dart';
import 'package:provider/provider.dart';

class ChangePassPage extends StatefulWidget {
  static const routeName = 'changePassPage';

  @override
  _ChangePassPageState createState() => _ChangePassPageState();
}

class _ChangePassPageState extends State<ChangePassPage> {
  ChangePassViewModel viewModel;

  @override
  Widget build(BuildContext context) {
    viewModel = Provider.of<ChangePassViewModel>(context);

    return Scaffold(
      backgroundColor: Color(0xFFF1EFF2),
      extendBodyBehindAppBar: true,
      appBar: AppBar(
        brightness: Brightness.light,
        elevation: 0,
        leading: new IconButton(
          icon: new Icon(
            Icons.arrow_back,
            color: Colors.white,
          ),
          onPressed: () => Navigator.of(context).pop(),
        ),
        backgroundColor: Colors.transparent,
      ),
      body: Container(
        decoration: BoxDecoration(
          image: DecorationImage(
              image: AssetImage("assets/bg.png"),
              fit: BoxFit.fitWidth,
              alignment: Alignment.topCenter),
        ),
        child: SafeArea(
          child: SingleChildScrollView(
            child: Column(
              children: [
                ButtonBar(
                  alignment: MainAxisAlignment.spaceEvenly,
                  children: [
                    FlatButton(
                        child: Text(
                          "แก้ไขรหัสผ่าน",
                          style: TextStyle(
                              color: Colors.white,
                              fontSize: 14,
                              fontWeight: FontWeight.w900),
                        ),
                        onPressed: () {})
                  ],
                ),
                Padding(
                  padding: const EdgeInsets.only(left: 16.0, right: 16.0),
                  child: Card(
                      clipBehavior: Clip.antiAlias,
                      elevation: 0,
                      shape: RoundedRectangleBorder(
                          borderRadius:
                          BorderRadius.all(Radius.circular(32.0))),
                      child: Padding(
                        padding:
                        const EdgeInsets.fromLTRB(24.0, 16.0, 24.0, 56.0),
                        child: Column(
                          children: [
                            TextFormField(
                              controller: viewModel.passwordController,
                              obscureText: true,
                              inputFormatters: [LengthLimitingTextInputFormatter(8),],
                              decoration: InputDecoration(
                                  labelText: "รหัสผ่านใหม่ (New Password)*",
                                  labelStyle:
                                  TextStyle(color: Color(0xFF767676)),
                                  focusedBorder: UnderlineInputBorder(
                                    borderSide:
                                    BorderSide(color: Color(0xFF352641)),
                                  ),
                                  enabledBorder: UnderlineInputBorder(
                                    borderSide:
                                    BorderSide(color: Color(0xFFDDDDDD)),
                                  )),
                            ),
                            TextFormField(
                              controller: viewModel.confirmPasswordController,
                              obscureText: true,
                              inputFormatters: [LengthLimitingTextInputFormatter(8),],
                              decoration: InputDecoration(
                                  labelText:
                                  "ยืนยันรหัสผ่านใหม่ (Comfirm Password)*",
                                  labelStyle:
                                  TextStyle(color: Color(0xFF767676)),
                                  focusedBorder: UnderlineInputBorder(
                                    borderSide:
                                    BorderSide(color: Color(0xFF352641)),
                                  ),
                                  enabledBorder: UnderlineInputBorder(
                                    borderSide:
                                    BorderSide(color: Color(0xFFDDDDDD)),
                                  )),
                            )
                          ],
                        ),
                      )),
                ),
                Padding(
                  padding: const EdgeInsets.fromLTRB(48, 40, 48, 32),
                  child: ButtonTheme(
                    minWidth: MediaQuery.of(context).size.width,
                    height: 48,
                    child: FlatButton(
                        child: Text(
                          "ยืนยัน",
                          style: TextStyle(color: Colors.white),
                        ),
                        onPressed: () {
                            viewModel.submitResetPassword().then((value) => {
                              showDialog(
                                context: context,
                                barrierDismissible: false,
                                builder: (BuildContext context) {
                                  return AlertDialog(
                                    title: Text('PEA'),
                                    content: const Text(
                                        'ทำการเปลี่ยนรหัสผ่านเรียบร้อยแล้ว'),
                                    actions: [
                                      FlatButton(
                                        child: Text('ตกลง'),
                                        onPressed: () {
                                          Navigator.of(context).pop();
                                          Navigator.of(context).pop();
                                        },
                                      ),
                                    ],
                                  );
                                },
                              )
                            },
                                onError: (e) => setState(() {
                                  try {
                                    showAlertDialog(
                                        context,
                                        e
                                            .toString()
                                            .split('Exception: ')
                                            .last);
                                  } catch (e) {
                                    print(e);
                                    showAlertDialog(context,
                                        "เกิดข้อผิดพลาด กรุณาลองอีกครั้งในภายหลัง");
                                  }
                                }));
                        },
                        color: Color(0xff8a56ac),
                        shape: new RoundedRectangleBorder(
                            borderRadius: new BorderRadius.circular(24.0))),
                  ),
                ),
              ],
            ),
          ),
        ),
      ),
    );
  }
}
