import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:loading_overlay/loading_overlay.dart';
import 'package:pea_cremation/shares/app_drawer.dart';
import 'package:pea_cremation/domain/model/dead_member.dart';
import 'package:pea_cremation/pages/dead/dead_page_view_model.dart';
import 'package:pea_cremation/shares/alert_dialog.dart';
import 'package:pea_cremation/shares/months.dart';
import 'package:provider/provider.dart';

class DeadPage extends StatefulWidget {
  static const routeName = "/deadPage";

  @override
  _DeadPageState createState() => _DeadPageState();
}

class _DeadPageState extends State<DeadPage> {
  DeadPageViewModel viewModel;

  @override
  Widget build(BuildContext context) {
    viewModel = Provider.of<DeadPageViewModel>(context);
    viewModel.init();

    return Scaffold(
      appBar: AppBar(
        elevation: 0,
        toolbarHeight: 120,
        title: SizedBox(
          width: double.infinity,
          child: Padding(
            padding: const EdgeInsets.only(left: 16, top: 24),
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                Text("รายชื่อสมาชิกที่ถึงแก่กรรม",
                    style: TextStyle(color: Colors.white, fontSize: 24)),
                Padding(
                  padding: const EdgeInsets.only(top: 8.0, bottom: 16.0),
                  child: Row(
                    children: getFilterBarViews(context),
                  )
                )
              ],
            ),
          ),
        ),
        backgroundColor: Color(0xFF7F3F97),
        leading: AppDrawer.appBarLeadIcon(context),
      ),
      drawer: AppDrawer().getAppDrawer(context),
      body: SafeArea(
        child: FutureBuilder<List<DeadMember>>(
            future: viewModel.deadMemberList,
            builder: (context, snapshot) {
              if (snapshot.hasData) {
                print('########## BUILD DEAD MEMBER LIST ##########');

                return NotificationListener<ScrollNotification>(
                    onNotification: (ScrollNotification scrollInfo) {
                      if (!viewModel.isLoadMore && scrollInfo.metrics.pixels ==
                          scrollInfo.metrics.maxScrollExtent) {
                        setState(() {
                          if(viewModel.isLoadMore) {
                            return;
                          }

                          viewModel.isLoadMore = true;
                          // viewModel.isLoading = true;
                          viewModel.loadMoreData();
                        });

                      }

                      return true;
                    },
                    child: LoadingOverlay(
                      isLoading: viewModel.isLoading,
                      color: Colors.black54,
                      child: ListView.builder(
                          itemCount: snapshot.data.length + 1,
                          itemBuilder: (context, index) {
                            if (index == 0) {
                              return ListTile(
                                title: getHeader(),
                              );
                            } else {
                              return ListTile(
                                contentPadding: const EdgeInsets.only(left:24,right:24),
                                title: getItem(index,
                                    snapshot.data[index-1]),
                              );
                            }
                          }
                      ),
                    )
                );
              } else {
                return Container();
              }
            }
        ),
      )
    );
  }

  Widget getItem(int no, DeadMember deadMember) {
    return Text('$no. ${deadMember.title}${deadMember.name} ${deadMember.surname}');
  }

  List<Widget> getFilterBarViews(BuildContext context) {
    return [
      Expanded(
        child: Theme(
          data: Theme.of(context).copyWith(canvasColor: Color(0xFF7F3F97)),
          child: DropdownButton<int>(
            value: viewModel.selectedMonth,
            isExpanded: true,
            underline: SizedBox(),
            hint: Text("เดือน", style: TextStyle(color: Colors.white60),),
            style: TextStyle(color: Colors.white60),
            onChanged: (int newValue) {
              setState(() {
                viewModel.selectedMonth = newValue;
              });
            },
            items: monthMap.keys
                .map<DropdownMenuItem<int>>((int value) {
              return DropdownMenuItem<int>(
                value: value,
                child: Center(child: Text('${monthMap[value]}'))
              );
            }).toList(),
          ),
        ),
      ),
      Expanded(
        child: Theme(
          data: Theme.of(context).copyWith(canvasColor: Color(0xFF7F3F97)),
          child: DropdownButton<int>(
            value: viewModel.selectedYear,
            isExpanded: true,
            underline: SizedBox(),
            style: TextStyle(color: Colors.white60),
            hint: Text("ปี", style: TextStyle(color: Colors.white60),),
            onChanged: (int newValue) {
              setState(() {
                viewModel.selectedYear = newValue;
              });
            },
            items: [for(var i=viewModel.minYear; i<=viewModel.currentYear; i++) i]
                .map<DropdownMenuItem<int>>((int value) {
              return DropdownMenuItem<int>(
                value: value,
                child: Center(child: Text('$value'))
              );
            }).toList(),
          ),
        ),
      ),
      IconButton(
        icon: Icon(Icons.search),
        color: Colors.white,
        onPressed: () {
          setState(() {
            try {
              viewModel.searchClick();
            } catch(e) {
              showAlertDialog(context, e.message);
            }
          });
        },
      )
    ];
  }

  Widget getHeader() {
    if (viewModel.totalCount == 0) {
      return Text('ไม่มีผู้เสียชีวิต', style: TextStyle(fontWeight: FontWeight.bold));
    } else {
      return Text('จำนวน ${viewModel.totalCount} ราย', style: TextStyle(fontWeight: FontWeight.bold));
    }
  }
}
