import 'package:flutter/material.dart';
import 'package:flutter_custom_tabs/flutter_custom_tabs.dart';
import 'package:pea_cremation/domain/model/file_link.dart';
import 'package:pea_cremation/pages/download/form_download_view_model.dart';
import 'package:pea_cremation/shares/app_drawer.dart';
import 'package:provider/provider.dart';

class FormDownloadPage extends StatefulWidget {
  static const routeName = "/formDownloadPage";

  @override
  _FormDownloadPageState createState() => _FormDownloadPageState();
}

class _FormDownloadPageState extends State<FormDownloadPage> {
  FormDownloadViewModel viewModel;

  @override
  Widget build(BuildContext context) {
    viewModel = Provider.of<FormDownloadViewModel>(context);
    viewModel.getFileList();

    return Scaffold(
      appBar: AppBar(
        elevation: 0,
        title: SizedBox(
          width: double.infinity,
          child: Text("ดาวโหลดแบบฟอร์ม",
              style: TextStyle(color: Colors.white, fontSize: 24)),
        ),
        backgroundColor: Color(0xFF7F3F97),
        leading: AppDrawer.appBarLeadIcon(context),
      ),
      drawer: AppDrawer().getAppDrawer(context),
      body: SafeArea(
          child: FutureBuilder<List<FileLink>>(
            future: viewModel.fileList,
            builder: (context, snapshot) {
              if (snapshot.hasData) {
                return ListView.builder(
                  itemCount: snapshot.data.length,
                  itemBuilder: (context, index) {
                    return ListTile(
                      title: getItem(
                          '${index + 1}. ${snapshot.data[index].name}'), //Text('${items[index]}'),
                      onTap: () {
                        print('tap');
                        _launchURL(context, snapshot.data[index].url);
                      },
                    );
                  }
                );
              } else {
                return Container();
              }
            }
          )),
    );
  }

  Widget getItem(String text) {
    return Padding(
      padding: const EdgeInsets.only(left:8, top: 8, bottom: 8),
      child: Row(
        children: [
          Expanded(
            child: Padding(
              padding: const EdgeInsets.only(left: 8.0, right: 8),
              child: Text('$text'),
            ),
          ),
          Padding(
            padding: const EdgeInsets.only(right:8.0),
            child: Image(
              image: AssetImage('assets/download.png'),
            ),
          )
        ],
      ),
    );
  }

  void _launchURL(BuildContext context, String url) async {
    try {
      print(url);
    await launch(
        Uri.encodeFull(url),
        option: new CustomTabsOption(
          toolbarColor: Color(0xff8a56ac),
          enableDefaultShare: true,
          enableUrlBarHiding: true,
          showPageTitle: true,
          animation: new CustomTabsAnimation.slideIn(),
        ),
      );
      // setState(() {
      //   viewModel.isLoading = false;
      // });
    } catch (e) {
      debugPrint(e.toString());
    }
  }
}
