import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:pea_cremation/pages/forgotpass/forgot_pass_page_arguments.dart';
import 'package:pea_cremation/pages/forgotpass/forgot_pass_page_view_model.dart';
import 'package:pea_cremation/shares/alert_dialog.dart';
import 'package:provider/provider.dart';

class ForgotPassPage extends StatefulWidget {
  static const routeName = 'ForgotPassPage';

  @override
  _ForgotPassPageState createState() => _ForgotPassPageState();
}

class _ForgotPassPageState extends State<ForgotPassPage> {
  ForgotPassPageViewModel viewModel;
  ForgotPassPageArguments args;

  @override
  Widget build(BuildContext context) {
    viewModel = Provider.of<ForgotPassPageViewModel>(context);
    args = ModalRoute.of(context).settings.arguments;

    viewModel.memberType = args.memberType;

    return Scaffold(
      backgroundColor: Color(0xFFF1EFF2),
      body: Container(
        decoration: BoxDecoration(
          image: DecorationImage(
              image: AssetImage("assets/bg.png"),
              fit: BoxFit.fitWidth,
              alignment: Alignment.topCenter),
        ),
        child: SafeArea(
          child: SingleChildScrollView(
            child: Column(
              children: [
                Padding(
                  padding: const EdgeInsets.only(top: 24.0),
                  child: ButtonBar(
                    alignment: MainAxisAlignment.spaceEvenly,
                    children: [
                      FlatButton(
                          child: Text(
                            "เข้าสู่ระบบ",
                            style: TextStyle(color: Colors.white, fontSize: 14),
                          ),
                          onPressed: () {
                            Navigator.pop(context);
                          }),
                      FlatButton(
                          child: Text(
                            "ลืมรหัสผ่าน",
                            style: TextStyle(
                                color: Colors.white,
                                fontSize: 14,
                                fontWeight: FontWeight.w900),
                          ),
                          onPressed: () {})
                    ],
                  ),
                ),
                Padding(
                  padding: const EdgeInsets.only(left: 16.0, right: 16.0),
                  child: Card(
                      clipBehavior: Clip.antiAlias,
                      elevation: 0,
                      shape: RoundedRectangleBorder(
                          borderRadius:
                              BorderRadius.all(Radius.circular(32.0))),
                      child: Padding(
                        padding:
                            const EdgeInsets.fromLTRB(24.0, 16.0, 24.0, 56.0),
                        child: Column(
                          children: [
                            Visibility(
                              visible: viewModel.step == 1,
                              child: TextFormField(
                                controller: viewModel.idCardController,
                                decoration: InputDecoration(
                                    labelText: "เลขประจำตัวประชาชน *",
                                    labelStyle:
                                        TextStyle(color: Color(0xFF767676)),
                                    focusedBorder: UnderlineInputBorder(
                                      borderSide:
                                          BorderSide(color: Color(0xFF352641)),
                                    ),
                                    enabledBorder: UnderlineInputBorder(
                                      borderSide:
                                          BorderSide(color: Color(0xFFDDDDDD)),
                                    )),
                              ),
                            ),
                            Visibility(
                              visible: viewModel.step == 1,
                              child: TextFormField(
                                controller: viewModel.emailController,
                                decoration: InputDecoration(
                                    labelText: "Email",
                                    labelStyle:
                                        TextStyle(color: Color(0xFF767676)),
                                    focusedBorder: UnderlineInputBorder(
                                      borderSide:
                                          BorderSide(color: Color(0xFF352641)),
                                    ),
                                    enabledBorder: UnderlineInputBorder(
                                      borderSide:
                                          BorderSide(color: Color(0xFFDDDDDD)),
                                    )),
                              ),
                            ),
                            Visibility(
                              visible: viewModel.step == 2,
                              child: TextFormField(
                                controller: viewModel.passwordController,
                                obscureText: true,
                                inputFormatters: [
                                  LengthLimitingTextInputFormatter(8),
                                ],
                                decoration: InputDecoration(
                                    labelText: "รหัสผ่านใหม่ (New Password)*",
                                    labelStyle:
                                        TextStyle(color: Color(0xFF767676)),
                                    focusedBorder: UnderlineInputBorder(
                                      borderSide:
                                          BorderSide(color: Color(0xFF352641)),
                                    ),
                                    enabledBorder: UnderlineInputBorder(
                                      borderSide:
                                          BorderSide(color: Color(0xFFDDDDDD)),
                                    )),
                              ),
                            ),
                            Visibility(
                              visible: viewModel.step == 2,
                              child: TextFormField(
                                controller: viewModel.confirmPasswordController,
                                obscureText: true,
                                inputFormatters: [
                                  LengthLimitingTextInputFormatter(8),
                                ],
                                decoration: InputDecoration(
                                    labelText:
                                        "ยืนยันรหัสผ่านใหม่ (Comfirm Password)",
                                    labelStyle:
                                        TextStyle(color: Color(0xFF767676)),
                                    focusedBorder: UnderlineInputBorder(
                                      borderSide:
                                          BorderSide(color: Color(0xFF352641)),
                                    ),
                                    enabledBorder: UnderlineInputBorder(
                                      borderSide:
                                          BorderSide(color: Color(0xFFDDDDDD)),
                                    )),
                              ),
                            )
                          ],
                        ),
                      )),
                ),
                Padding(
                  padding: const EdgeInsets.fromLTRB(48, 40, 48, 32),
                  child: ButtonTheme(
                    minWidth: MediaQuery.of(context).size.width,
                    height: 48,
                    child: FlatButton(
                        child: Text(
                          viewModel.step == 1 ? "ต่อไป" : "ยืนยัน",
                          style: TextStyle(color: Colors.white),
                        ),
                        onPressed: () {
                          if (viewModel.step == 1) {
                            viewModel.checkMember().then(
                                (value) => {
                                      setState(() {
                                        viewModel.step = 2;
                                      })
                                    },
                                onError: (e) => setState(() {
                                      try {
                                        showAlertDialog(
                                            context,
                                            json.decode(e
                                                .toString()
                                                .split('Exception: ')
                                                .last)["message"]);
                                      } catch (e2) {
                                        try {
                                          print(e);
                                          showAlertDialog(
                                              context,e.toString().split('Exception: ').last
                                          );
                                        } catch (e) {
                                          showAlertDialog(context,
                                              "เกิดข้อผิดพลาด กรุณาลองอีกครั้งในภายหลัง");
                                        }
                                      }
                                    }));
                          } else {
                            viewModel.submitResetPassword().then(
                                (value) => {
                                      showDialog(
                                        context: context,
                                        barrierDismissible: false,
                                        builder: (BuildContext context) {
                                          return AlertDialog(
                                            title: Text('PEA'),
                                            content: const Text(
                                                'ทำการ Reset รหัสผ่านเรียบร้อยแล้ว'),
                                            actions: [
                                              FlatButton(
                                                child: Text('ตกลง'),
                                                onPressed: () {
                                                  Navigator.of(context).pop();
                                                  Navigator.of(context).pop();
                                                },
                                              ),
                                            ],
                                          );
                                        },
                                      )
                                    },
                                onError: (e) => setState(() {
                                      try {
                                        showAlertDialog(
                                            context,
                                            e
                                                .toString()
                                                .split('Exception: ')
                                                .last);
                                      } catch (e) {
                                        print(e);
                                        showAlertDialog(context,
                                            "เกิดข้อผิดพลาด กรุณาลองอีกครั้งในภายหลัง");
                                      }
                                    }));
                          }
                        },
                        color: Color(0xff8a56ac),
                        shape: new RoundedRectangleBorder(
                            borderRadius: new BorderRadius.circular(24.0))),
                  ),
                ),
                Text(
                  "ในกรณีที่ไม่สามารถ Reset รหัสผ่านได้\nกรุณาติดต่อ 02-590-5227\nหรือ 02-590-9489",
                  textAlign: TextAlign.center,
                )
              ],
            ),
          ),
        ),
      ),
    );
  }
}
