
import 'package:flutter/material.dart';
import 'package:pea_cremation/domain/model/user.dart';
import 'package:pea_cremation/domain/service/user_service.dart';
import 'package:pea_cremation/shares/form_validator.dart';

class ForgotPassPageViewModel extends ChangeNotifier {

  UserService userService = UserService();

  var loading = false;
  var step = 1;
  var memberType = 1;
  var memberCode = '';

  var idCardController = TextEditingController();
  var emailController = TextEditingController();
  var passwordController = TextEditingController();
  var confirmPasswordController = TextEditingController();

  Future<User> checkMember() async {
    if(idCardController.text.trim() == '') {
      throw Exception('กรุณากรอกเลขประจำตัวประชาชน');
    } else if(invalidIdCard(idCardController.text)) {
      throw Exception('เลขบัตรประจำตัวประชาชนไม่ถูกต้อง');
    }

    User user = await userService.checkMember(idCardController.text, emailController.text, memberType);

    memberCode = user.memberCode;

    if (memberCode != '') {
      step = 2;
    }

    return user;
  }

  Future<Map<String, dynamic>> submitResetPassword() async {
    if (invalidInput()) {
      throw Exception();
    }
    print("Passs");
    var response =  userService.resetPassword(memberCode, passwordController.text, confirmPasswordController.text, memberType);

    return response;
  }

  bool invalidInput() {
    if(passwordController.text != confirmPasswordController.text) {
      throw Exception('กรุณากรอกรหัสผ่านให้ตรงกัน');
    } else if(passwordController.text.length < 8) {
      throw Exception('กรุณากำหนดรหัสผ่าน 8 หลัก');
    } else if(!validFormatPassword(passwordController.text)) {
      throw Exception('กรุณากำหนดรหัสผ่านให้ประกอบไปด้วยตัวเลขตัวอักษร และอักขระ');
    } else if(notValidEmail(emailController.text)) {
      throw Exception('กรุณากรอก email ให้ถูกต้อง');
    }

    return false;
  }

  bool notValidEmail(String email) {
    return !RegExp(r"^[a-zA-Z0-9.a-zA-Z0-9.!#$%&'*+-/=?^_`{|}~]+@[a-zA-Z0-9]+\.[a-zA-Z]+").hasMatch(email);
  }

  bool validFormatPassword(String password) {
    var hasCharacters = RegExp('[a-zA-Z]').hasMatch(password);
    var hasNumbers = RegExp('[0-9]').hasMatch(password);
    var hasSymbol = RegExp(r'[!@#$%^&*(),.?":{}|<>]').hasMatch(password);

    return hasCharacters && hasNumbers && hasSymbol;
  }
}
