import 'package:flutter/material.dart';
import 'package:pea_cremation/domain/model/bill.dart';
import 'package:pea_cremation/domain/model/invoice.dart';
import 'package:pea_cremation/domain/model/receipt.dart';
import 'package:pea_cremation/domain/model/user.dart';
import 'package:pea_cremation/pages/invoice/invoice/invoice_view_page_arguments.dart';
import 'package:pea_cremation/pages/invoice/invoice_download_view_model.dart';
import 'package:pea_cremation/pages/invoice/receipt/receipt_view_page.dart';
import 'package:pea_cremation/pages/invoice/receipt/receipt_view_page_arguments.dart';
import 'package:pea_cremation/shares/alert_dialog.dart';
import 'package:pea_cremation/shares/app_drawer.dart';
import 'package:pea_cremation/shares/months.dart';
import 'package:provider/provider.dart';

import 'invoice/invoice_view_page.dart';

class InvoiceDownloadPage extends StatefulWidget {
  static const routeName = "/invoiceDownloadPage";

  @override
  _InvoiceDownloadPageState createState() => _InvoiceDownloadPageState();
}

class _InvoiceDownloadPageState extends State<InvoiceDownloadPage> {
  InvoiceDownloadViewModel viewModel;

  @override
  Widget build(BuildContext context) {
    viewModel = Provider.of<InvoiceDownloadViewModel>(context);
    viewModel.getUser();
    viewModel.searchClick();

    return Scaffold(
      appBar: AppBar(
        elevation: 0,
        title: SizedBox(
          width: double.infinity,
          child: Padding(
            padding: const EdgeInsets.only(left: 24.0),
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                Text("ข้อมูลการชำระเงิน",
                    style: TextStyle(color: Colors.white, fontSize: 24))
              ],
            ),
          ),
        ),
        backgroundColor: Color(0xFF7F3F97),
      ),
      drawer: AppDrawer().getAppDrawer(context),
      body: SafeArea(
          child: Column(
        children: [
          Container(color: Color(0xFF7F3F97), child: getUserInfo(context)),
          getFilterBarView(context),
          Expanded(
              child: FutureBuilder<List<Bill>>(
                  future: viewModel.receiptList,
                  builder: (context, snapshot) {
                    if (snapshot.hasData) {
                      return ListView.builder(
                        itemCount: snapshot.data.length,
                        itemBuilder: (context, index) {
                          return ListTile(
                            title: getItem(
                                index,
                                snapshot
                                    .data[index]), //Text('${items[index]}'),
                          );
                        },
                      );
                    } else {
                      return Container();
                    }
                  }))
        ],
      )),
    );
  }

  Widget getUserInfo(BuildContext context) {
    return Padding(
      padding: const EdgeInsets.only(left: 16.0),
      child: FutureBuilder<User>(
        future: viewModel.user,
        builder: (context, snapshot) {
          if(snapshot.hasData) {
            return Row(
                mainAxisAlignment: MainAxisAlignment.start,
                crossAxisAlignment: CrossAxisAlignment.start,
                children: [
                  Padding(
                    padding: const EdgeInsets.only(bottom: 16.0),
                    child: snapshot.data.coverImagePath == null ?  Container(
                        width: 64.0,
                        height: 64.0,
                        decoration: new BoxDecoration(
                            shape: BoxShape.circle,
                            image: new DecorationImage(
                                fit: BoxFit.fill,
                                image: AssetImage('assets/user@2x.png')
                            )
                        )
                    ) : Container(
                        width: 64.0,
                        height: 64.0,
                        decoration: new BoxDecoration(
                            shape: BoxShape.circle,
                            image: new DecorationImage(
                                fit: BoxFit.fill,
                                image: NetworkImage(snapshot.data.coverImagePath)
                            )
                        )
                    )
                  ),
                  Padding(
                    padding: const EdgeInsets.only(left: 16.0),
                    child: Column(
                      crossAxisAlignment: CrossAxisAlignment.start,
                      children: [
                        Text("${snapshot.data.firstName} ${snapshot.data.lastName}",
                            style: TextStyle(
                                fontSize: 22,
                                fontWeight: FontWeight.bold,
                                color: Colors.white)),
                        Text(getMemberCodeText(snapshot.data),
                            style: TextStyle(fontSize: 12, color: Colors.white70))
                      ],
                    ),
                  )
                ]);
          } else {
            return Container(color: Colors.transparent);
          }
        }
      ),
    );
  }

  Widget getFilterBarView(BuildContext context) {
    return Row(children: [
      Expanded(
        child: DropdownButton<int>(
          value: viewModel.selectedMonth,
          isExpanded: true,
          underline: SizedBox(),
          hint: Text("เดือน"),
          onChanged: (int newValue) {
            setState(() {
              viewModel.selectedMonth = newValue;
            });
          },
          items: monthMap.keys.map<DropdownMenuItem<int>>((int value) {
            return DropdownMenuItem<int>(
                value: value, child: Center(child: Text('${monthMap[value]}')));
          }).toList(),
        ),
      ),
      Expanded(
        child: DropdownButton<int>(
          value: viewModel.selectedYear,
          isExpanded: true,
          underline: SizedBox(),
          hint: Text("ปี"),
          onChanged: (int newValue) {
            setState(() {
              viewModel.selectedYear = newValue;
            });
          },
          items: [
            for (var i = viewModel.minYear; i <= viewModel.currentYear; i++) i
          ].map<DropdownMenuItem<int>>((int value) {
            return DropdownMenuItem<int>(
                value: value, child: Center(child: Text('$value')));
          }).toList(),
        ),
      ),
      IconButton(
        icon: Icon(Icons.search),
        color: Colors.black87,
        onPressed: () {
          setState(() {
            try {
              viewModel.searchClick();
            } catch (e) {
              showAlertDialog(context, e.message);
            }
          });
        },
      )
    ]);
  }

  Widget getItem(int index, Bill data) {
    return Padding(
      padding: const EdgeInsets.only(left: 8, top: 8, bottom: 8),
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          Row(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              Padding(
                padding: const EdgeInsets.only(top: 8.0),
                child: Text('${index + 1}.'),
              ),
              if(data.receipt != null) InkWell(
                child: Padding(
                  padding: const EdgeInsets.all(8.0),
                  child: Text('ใบเสร็จรับเงิน', style: TextStyle(color: Colors.indigoAccent)),
                ),
                onTap: () { _openReceipt(data.receipt); },
              ),
              if(data.receipt != null) InkWell(
                child: Padding(
                  padding: const EdgeInsets.only(top:16,left: 8),
                  child: Image(
                    image: AssetImage('assets/download@2x.png'),
                    width: 24,
                    height: 24,
                  ),
                ),
                onTap: () { _openReceipt(data.receipt); },
              ),
              if(data.invoice != null) InkWell(
                child: Padding(
                  padding: const EdgeInsets.all(8.0),
                  child: Text('ใบแจ้งหนี้', style: TextStyle(color: Colors.indigoAccent)),
                ),
                onTap: () { _openInvoice(data.invoice); },
              ),
              if(data.invoice != null) InkWell(
                child: Padding(
                  padding: const EdgeInsets.only(top:16,left: 8),
                  child: Image(
                    image: AssetImage('assets/download@2x.png'),
                    width: 24,
                    height: 24,
                  ),
                ),
                onTap: () { _openInvoice(data.invoice); },
              )
            ],
          ),
          Padding(
            padding: const EdgeInsets.only(left:16.0),
            child: Text('${monthMap[int.parse(data.month)]} ${data.year}', style: TextStyle(color: Colors.black54)),
          )
        ],
      ),
    );
  }

  void _openReceipt(Receipt receipt) {
    Navigator.pushNamed(context, ReceiptViewPage.routeName,arguments:
    ReceiptViewPageArguments(viewModel.selectedMonth, viewModel.selectedYear, receipt));
  }

  void _openInvoice(Invoice invoice) {
    Navigator.pushNamed(context, InvoiceViewPage.routeName,arguments:
    InvoiceViewPageArguments(invoice.month, invoice.year));
  }
}
