import 'package:flutter/material.dart';
import 'package:pea_cremation/domain/model/bill.dart';
import 'package:pea_cremation/domain/model/invoice.dart';
import 'package:pea_cremation/domain/model/receipt.dart';
import 'package:pea_cremation/domain/model/user.dart';
import 'package:pea_cremation/domain/service/base_service.dart';
import 'package:pea_cremation/domain/service/payment_service.dart';
import 'package:pea_cremation/shares/app_drawer.dart';

class InvoiceDownloadViewModel extends ChangeNotifier {
  PaymentService paymentService = PaymentService();
  BaseService baseService = BaseService();

  int minYear = 2563;
  int currentYear = (DateTime
      .now()
      .year + 543);
  int selectedYear = (DateTime
      .now()
      .year + 543);

  int selectedMonth = 99;

  bool isLoading = false;

  Future<List<Bill>> receiptList;
  Future<User> user;

  void searchClick() {
    receiptList = getReceiptInvoiceHistory(selectedMonth, selectedYear);
  }

  Future<User> getUser() {
    user = baseService.loadUser();

    return user;
  }

  Future<List<Bill>> getReceiptInvoiceHistory(int selectedMonth, int selectedYear) async {
    var results = await paymentService.getReceiptInvoiceHistory(selectedMonth, selectedYear);

    var data = results['dataReceipt'] as List;
    var data2 = results['dataInvoice'] as List;

    return buildBillList(data.map((json) => Receipt.fromJson(json)).toList(), data2.map((json) => Invoice.fromJson(json)).toList());
  }

  List<Bill> buildBillList(List<Receipt> receiptList, List<Invoice> invoiceList) {
    var billList = List<Bill>();

    receiptList.forEach((element) {
      billList.add(Bill(
        receipt: element,
        invoice: null,
        month: element.payDate.substring(4, 6),
        year: element.payDate.substring(0, 4)
      ));
    });

    invoiceList.forEach((element)  {
      var index = billList.indexWhere((it) => it.month == element.month && it.year == element.year);

      if (index > -1) {
        billList[index].invoice = element;
      } else {
        billList.add(Bill(
            receipt: null,
            invoice: element,
            month: element.month,
            year: element.year
        ));
      }
    });

    billList.sort((a, b) => ("${a.year}${a.month}".compareTo("${b.year}${b.month}")));

    return billList;
  }
}