import 'dart:io';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:image_gallery_saver/image_gallery_saver.dart';
import 'package:pea_cremation/domain/model/receipt_info.dart';
import 'package:pea_cremation/pages/invoice/receipt/receipt_view_model.dart';
import 'package:pea_cremation/pages/invoice/receipt/receipt_view_page_arguments.dart';
import 'package:pea_cremation/shares/alert_dialog.dart';
import 'package:permission_handler/permission_handler.dart';
import 'package:provider/provider.dart';
import 'package:screenshot/screenshot.dart';

class ReceiptViewPage extends StatefulWidget {
  static const routeName = 'receiptViewPage';

  @override
  _ReceiptViewPageState createState() => _ReceiptViewPageState();
}

class _ReceiptViewPageState extends State<ReceiptViewPage> {
  ReceiptViewModel viewModel;
  ReceiptViewPageArguments args;

  @override
  Widget build(BuildContext context) {
    viewModel = Provider.of<ReceiptViewModel>(context);
    args = ModalRoute.of(context).settings.arguments;

    ScreenshotController screenshotController = ScreenshotController();

    viewModel.month = args.month;
    viewModel.year = args.year;
    viewModel.receipt = args.receipt;

    return Scaffold(
        backgroundColor: Color(0xFFF1EFF2),
        // extendBodyBehindAppBar: true,
        appBar: AppBar(
          elevation: 0,
          title: SizedBox(
            width: double.infinity,
            child: Text("ข้อมูลการชำระเงิน",
                style: TextStyle(color: Colors.white, fontSize: 24)),
          ),
          leading: new IconButton(
            icon: new Icon(
              Icons.arrow_back,
              color: Colors.white,
            ),
            onPressed: () => Navigator.of(context).pop(),
          ),
          actions: <Widget>[
            Padding(
                padding: EdgeInsets.only(right: 20.0),
                child: GestureDetector(
                  onTap: () {
                    // _incrementCounter();
                    // _imageFile = null;
                    screenshotController
                        .capture(delay: Duration(milliseconds: 10))
                        .then((File image) async {
                      var status = await Permission.storage.request();

                      print("Capture Done -> $status");
                      if (status.isGranted) {
                        setState(() {
                          viewModel
                              .saveImage(image)
                              .then((value) => _showCompleteDialog(context, value));
                        });
                      } else {
                        showDialog(
                            context: context,
                            builder: (BuildContext context) => AlertDialog(
                              title: Text('PEA'),
                              content: Text(
                                  'แอปพลิเคชันต้องการสิทธิ์ในการบันทึกภาพเข้าสู่คลังภาพ'),
                              actions: <Widget>[
                                FlatButton(
                                  child: Text("ไม่อนุญาต"),
                                  onPressed:  () => Navigator.of(context).pop(),
                                ),
                                FlatButton(
                                  child: Text('ตั้งค่า'),
                                  onPressed: () => openAppSettings(),
                                ),
                              ],
                            ));
                      }
                    }).catchError((onError) {
                      print(onError);
                    });
                  },
                  child: Icon(
                    Icons.file_download,
                    size: 26.0,
                  ),
                ))
          ],
          backgroundColor: Color(0xFF7F3F97),
        ),
        body: FutureBuilder<ReceiptInfo>(
            future: viewModel.getReceiptData(),
            builder: (context, snapshot) {
              if (snapshot.hasData) {
                return Screenshot(
                  controller: screenshotController,
                  child: Container(
                    // color: Colors.white,
                    decoration: BoxDecoration(
                      color: Colors.white,
                      image: DecorationImage(
                        image: AssetImage("assets/watermark.jpg"),
                        colorFilter: new ColorFilter.mode(Colors.black.withOpacity(0.2), BlendMode.dstATop),
                        fit: BoxFit.contain,

                      ),
                    ),
                    child: Padding(
                      padding: const EdgeInsets.all(24.0),
                      child: Column(
                        crossAxisAlignment: CrossAxisAlignment.start,
                        children: [
                          Padding(
                            padding:
                                const EdgeInsets.only(top: 16.0, bottom: 32.0),
                            child: Row(
                              mainAxisAlignment: MainAxisAlignment.start,
                              children: [
                                Expanded(
                                    child: Text(
                                  '${snapshot.data.memberName}',
                                  style: TextStyle(fontWeight: FontWeight.bold),
                                )),
                                Text(
                                  'เลขที่สมาชิก    ${snapshot.data.memberNo}',
                                  style: TextStyle(fontWeight: FontWeight.bold),
                                )
                              ],
                            ),
                          ),
                          Text(
                              'ใบเสร็จรับเงิน เลขที่    ${snapshot.data.receiptNo}'),
                          Padding(
                            padding:
                                const EdgeInsets.only(top: 16.0, bottom: 32.0),
                            child: Text(
                                'วันที่    ${snapshot.data.receiptDateFull}'),
                          ),
                          Padding(
                            padding:
                                const EdgeInsets.only(top: 8.0, bottom: 8.0),
                            child: Text('รายการ'),
                          ),
                          snapshot.data.deadAllowance == ""
                              ? Container()
                              : Padding(
                                  padding: const EdgeInsets.only(top: 16.0),
                                  child: Text('เงินสงเคราะห์'),
                                ),
                          snapshot.data.deadAllowance == ""
                              ? Container()
                              : Padding(
                                  padding: const EdgeInsets.only(top: 8.0),
                                  child: Row(
                                    children: [
                                      Text(
                                          'ศพที่ ${snapshot.data.deadAllowance}'),
                                      Expanded(
                                          child: Padding(
                                        padding: const EdgeInsets.only(
                                            left: 16.0, right: 16.0),
                                        child: Text(
                                          '${snapshot.data.allowance == "" ? "-" : viewModel.numberFormat.format(int.parse(snapshot.data.allowance))}',
                                          textAlign: TextAlign.right,
                                        ),
                                      )),
                                      Text('บาท')
                                    ],
                                  ),
                                ),
                          snapshot.data.deadDebtAllowance == ""
                              ? Container()
                              : Padding(
                                  padding: const EdgeInsets.only(top: 24.0),
                                  child: Text('เงินสงเคราะห์ค้างจ่าย'),
                                ),
                          snapshot.data.deadDebtAllowance == ""
                              ? Container()
                              : Padding(
                                  padding: const EdgeInsets.only(top: 8.0),
                                  child: Row(
                                    children: [
                                      Text(
                                          'ศพที่ ${snapshot.data.deadDebtAllowance}'),
                                      Expanded(
                                          child: Padding(
                                        padding: const EdgeInsets.only(
                                            left: 16.0, right: 16.0),
                                        child: Text(
                                            '${snapshot.data.debtAllowance == "" ? "-" : viewModel.numberFormat.format(int.parse(snapshot.data.debtAllowance))}',
                                            textAlign: TextAlign.right),
                                      )),
                                      Text('บาท')
                                    ],
                                  ),
                                ),
                          snapshot.data.deadForwardAllowance == ""
                              ? Container()
                              : Padding(
                                  padding: const EdgeInsets.only(top: 24.0),
                                  child: Text('เงินสงเคราะห์ล่วงหน้า'),
                                ),
                          snapshot.data.deadForwardAllowance == ""
                              ? Container()
                              : Padding(
                                  padding: const EdgeInsets.only(top: 8.0),
                                  child: Row(
                                    children: [
                                      Text(
                                          'ศพที่ ${snapshot.data.deadForwardAllowance == "" ? "-" : snapshot.data.deadForwardAllowance}'),
                                      Expanded(
                                          child: Padding(
                                        padding: const EdgeInsets.only(
                                            left: 16.0, right: 16.0),
                                        child: Text(
                                            '${snapshot.data.forwardAllowance == "" ? "-" : viewModel.numberFormat.format(int.parse(snapshot.data.forwardAllowance))}',
                                            textAlign: TextAlign.right),
                                      )),
                                      Text('บาท')
                                    ],
                                  ),
                                ),
                          Padding(
                            padding: const EdgeInsets.only(top: 32.0),
                            child: Row(
                              children: [
                                getHeaderText(snapshot.data.total),
                                Expanded(
                                    child: Padding(
                                      padding: const EdgeInsets.only(left:16.0, right:16.0),
                                      child: Text('${snapshot.data.total == ""?"-":viewModel.numberFormat.format(int.parse(snapshot.data.total).abs())}', textAlign: TextAlign.right),
                                    )),
                                Text('บาท')
                              ],
                            ),
                          )
                        ],
                      ),
                    ),
                  ),
                );
              } else {
                return Container();
              }
            }));
  }

  _showCompleteDialog(BuildContext context, dynamic value) {
    print(value);

    if (value['isSuccess']) {
      showAlertDialog(context, "บันทึกใบเสร็จไปที่คลังภาพเรียบร้อยแล้ว");
    }
  }

  Widget getHeaderText(String total) {
    try {
      if(total != "" && int.parse(total) > 0)
        return Text('รวมเป็นเงิน');
      else
        return Text('คงเหลือ');
    }catch(e) {
      return Text('รวมเป็นเงิน');
    }
  }
}
