import 'package:carousel_slider/carousel_slider.dart';
import 'package:flutter/material.dart';
import 'package:pea_cremation/domain/model/user.dart';
import 'package:pea_cremation/domain/service/user_service.dart';
import 'package:pea_cremation/pages/landing/landing_page_view_model.dart';
import 'package:pea_cremation/pages/login/login_page.dart';
import 'package:pea_cremation/pages/news/news_page.dart';
import 'package:pea_cremation/pea_cremation.dart';
import 'package:provider/provider.dart';

class LandingPage extends StatefulWidget {
  static const routeName = '/landingPage';

  @override
  _LandingPageState createState() => _LandingPageState();
}

class _LandingPageState extends State<LandingPage> {
  @override
  Widget build(BuildContext context) {
    final viewModel = Provider.of<LandingPageViewModel>(context);

    UserService().clearUserData();

    return Scaffold(
        backgroundColor: Colors.white,
        body: Container(
          child: SingleChildScrollView(
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.center,
              mainAxisAlignment: MainAxisAlignment.start,
              children: [
                Container(
                    height: MediaQuery.of(context).size.height * 0.6,
                    width: MediaQuery.of(context).size.width,
                    color: Colors.transparent,
                    child: SizedBox(
                      width: double.infinity,
                      height: double.infinity,
                      child: CarouselSlider(
                        options: CarouselOptions(
                          height: MediaQuery.of(context).size.height * 0.6,
                          viewportFraction: 1.0,
                          enlargeCenterPage: false,
                          autoPlay: true,
                        ),
                        items: viewModel.imageList
                            .map((item) => Container(
                                  child: Center(
                                      child: Image(
                                    image: AssetImage(item),
                                    fit: BoxFit.fitHeight,
                                    height: MediaQuery.of(context).size.height * 0.6,
                                    alignment: Alignment.bottomCenter,
                                  )),
                                ))
                            .toList(),
                      ),
                    )
                    // Image(
                    //   image: AssetImage("assets/img@3x.png"),
                    //   fit: BoxFit.fitWidth,
                    //   alignment: Alignment.bottomCenter,
                    // )
                    ),
                Padding(
                  padding: const EdgeInsets.only(top: 16.0, bottom: 16.0),
                  child: Row(
                    mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      Image(
                        image: AssetImage("assets/logo_app_3@3x.png"),
                        width: 80,
                        height: 80,
                        fit: BoxFit.contain,
                      ),
                      Padding(
                        padding: const EdgeInsets.only(top: 16.0),
                        child: Column(children: [
                          Text("แผนกฌาปนกิจสงเคราะห์",
                              textAlign: TextAlign.center,
                              style: TextStyle(
                                  fontWeight: FontWeight.bold,
                                  fontSize: 14,
                                  color: Color.fromARGB(255, 163, 169, 175))),
                          Padding(
                              padding: EdgeInsets.only(top: 8.0, bottom: 8.0),
                              child: Text(
                                "ยินดีต้อนรับ",
                                textAlign: TextAlign.center,
                                style: TextStyle(
                                    fontWeight: FontWeight.bold, fontSize: 32),
                              )),
                          Text("สู่ Application ฌาปนกิจสงเคราะห์",
                              textAlign: TextAlign.center,
                              style: TextStyle(
                                  fontWeight: FontWeight.bold,
                                  fontSize: 14,
                                  color: Color.fromARGB(255, 163, 169, 175))),
                        ]),
                      ),
                      Image(
                        image: AssetImage("assets/logo@3x.png"),
                        width: 80,
                        height: 80,
                        fit: BoxFit.contain,
                      )
                    ],
                  ),
                ),
                Padding(
                  padding: const EdgeInsets.all(24.0),
                  child: Center(
                    child: FutureBuilder<User>(
                      future: viewModel.verifyToken(),
                      builder: (context, snapshot) {
                        if(snapshot.hasData) {
                          viewModel.verifyToken().then((value) => {
                            Navigator.pushNamedAndRemoveUntil(context, NewsPage.routeName, (Route<dynamic> route) => false)
                          });

                          return Container();
                        } else {
                          return RaisedButton(
                            onPressed: () {
                              Navigator.pushNamedAndRemoveUntil(
                                  context,
                                  LoginPage.routeName,
                                      (Route<dynamic> route) => false);
                            },
                            child: const Text(
                                'NEXT', style: TextStyle(fontSize: 14)),
                            color: Color(0xff8a56ac),
                            textColor: Colors.white,
                            shape: RoundedRectangleBorder(
                                borderRadius: BorderRadius.circular(32.0)),
                          );
                        }
                      }
                    ),
                  ),
                )
              ],
            ),
          ),
        ));
  }
}
