import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:loading_overlay/loading_overlay.dart';
import 'package:pea_cremation/domain/model/user.dart';
import 'package:pea_cremation/pages/forgotpass/forgot_pass_page.dart';
import 'package:pea_cremation/pages/forgotpass/forgot_pass_page_arguments.dart';
import 'package:pea_cremation/pages/login/login_page_view_model.dart';
import 'package:pea_cremation/pages/news/news_page.dart';
import 'package:pea_cremation/pages/register/register_page_arguments.dart';
import 'package:pea_cremation/pages/register/register_page.dart';
import 'package:pea_cremation/shares/alert_dialog.dart';
import 'package:provider/provider.dart';

class LoginPage extends StatefulWidget {
  static const routeName = '/loginPage';

  @override
  _LoginPageState createState() => _LoginPageState();
}

class _LoginPageState extends State<LoginPage> {
  LoginPageViewModel viewModel;

  @override
  void dispose() {
    super.dispose();
  }

  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    viewModel = Provider.of<LoginPageViewModel>(context);

    return Scaffold(
      backgroundColor: Colors.white,
      resizeToAvoidBottomPadding: false,
      body: SafeArea(
        child: LoadingOverlay(
          isLoading: viewModel.isLoading,
          child: SafeArea(
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.center,
              mainAxisAlignment: MainAxisAlignment.start,
              children: [
                Row(
                  mainAxisAlignment: MainAxisAlignment.center,
                  children: [
                    Image(
                      image: AssetImage("assets/logo_app_3@3x.png"),
                      width: 64,
                      alignment: Alignment.center,
                    ),
                  ],
                ),
                Padding(
                    padding: const EdgeInsets.all(16.0),
                    child: Text("เลือกประเภทสมาชิก เข้าสู่ระบบ",
                        textAlign: TextAlign.center,
                        style: TextStyle(
                            fontWeight: FontWeight.bold, fontSize: 22))),
                ButtonBar(
                  alignment: MainAxisAlignment.spaceAround,
                  children: [
                    FlatButton(
                        child: Text(
                          "พนักงาน",
                          style: TextStyle(
                              color: viewModel.memberType == 1
                                  ? Colors.white
                                  : Color(0xffada5b5)),
                        ),
                        onPressed: () {
                          setState(() {
                            viewModel.memberType = 1;
                          });
                        },
                        color: viewModel.memberType == 1
                            ? Color(0xff8a56ac)
                            : Colors.transparent,
                        shape: new RoundedRectangleBorder(
                            borderRadius: new BorderRadius.circular(24.0))),
                    FlatButton(
                        child: Text(
                          "คู่สมรสพนักงาน",
                          style: TextStyle(
                              color: viewModel.memberType == 2
                                  ? Colors.white
                                  : Color(0xffada5b5)),
                        ),
                        onPressed: () {
                          setState(() {
                            viewModel.memberType = 2;
                          });
                        },
                        color: viewModel.memberType == 2
                            ? Color(0xff8a56ac)
                            : Colors.transparent,
                        shape: new RoundedRectangleBorder(
                            borderRadius: new BorderRadius.circular(24.0))),
                    FlatButton(
                        child: Text(
                          "อดีตพนักงาน\nและคู่สมรส",
                          textAlign: TextAlign.center,
                          style: TextStyle(
                              color: viewModel.memberType == 3
                                  ? Colors.white
                                  : Color(0xffada5b5)),
                        ),
                        padding: const EdgeInsets.all(8.0),
                        onPressed: () {
                          setState(() {
                            viewModel.memberType = 3;
                          });
                        },
                        color: viewModel.memberType == 3
                            ? Color(0xff8a56ac)
                            : Colors.transparent,
                        shape: new RoundedRectangleBorder(
                            borderRadius: new BorderRadius.circular(24.0)))
                  ],
                ),
                Column(
                  children: viewModel.memberType == 1
                      ? type1UserForm()
                      : type2Or3UserForm(),
                ),
                Padding(
                  padding: const EdgeInsets.fromLTRB(48, 40, 48, 32),
                  child: ButtonTheme(
                    minWidth: MediaQuery.of(context).size.width,
                    height: 48,
                    child: FlatButton(
                        child: Text(
                          "เข้าสู่ระบบ",
                          style: TextStyle(color: Colors.white),
                        ),
                        onPressed: () {
                          setState(() {
                            viewModel.isLoading = true;
                          });
                          viewModel.login().then(
                              (user) => {
                                    setState(() {
                                      // viewModel.isLoading = false;

                                      if (user.isApplyPolicy == '1') {
                                        openMainPage();
                                      } else {
                                        viewModel.getPolicy().then((value) => {
                                              showDialog(
                                                  context: context,
                                                  builder:
                                                      (BuildContext context) {
                                                    return AlertDialog(
                                                      title: Text("Policy"),
                                                      content:
                                                          SingleChildScrollView(
                                                              child: Text(value)),
                                                      actions: [
                                                        FlatButton(
                                                          child:
                                                              Text("ไม่ยอมรับ"),
                                                          onPressed: () {
                                                            viewModel
                                                                .setNotAgreeToPolicy();
                                                            Navigator.of(context)
                                                                .pop();
                                                          },
                                                        ),
                                                        FlatButton(
                                                          child: Text("ยอมรับ"),
                                                          onPressed: () {
                                                            print('onPressed()');
                                                            viewModel
                                                                .setAgreeToPolicy()
                                                                .then(
                                                                    (value) => {
                                                                          setState(
                                                                              () {
                                                                            print(
                                                                                'value = $value');
                                                                            if (value) {
                                                                              Navigator.of(context).pop();
                                                                              openMainPage();
                                                                            }
                                                                          })
                                                                        },
                                                                    onError: (e) {
                                                              print(e);
                                                            });
                                                          },
                                                        )
                                                      ],
                                                    );
                                                  })
                                            });
                                      }
                                    })
                                  }, onError: (e) {
                            setState(() {
                              try {
                                viewModel.isLoading = false;
                                showAlertDialog(
                                    context,
                                    json.decode(e
                                        .toString()
                                        .split('Exception: ')
                                        .last)["message"]);
                              } catch (e) {
                                print(e);
                                showAlertDialog(context,
                                    "เกิดข้อผิดพลาด กรุณาลองอีกครั้งในภายหลัง");
                              }
                            });
                          });
                        },
                        color: Color(0xff8a56ac),
                        shape: new RoundedRectangleBorder(
                            borderRadius: new BorderRadius.circular(24.0))),
                  ),
                ),
                ButtonBar(
                  alignment: MainAxisAlignment.spaceEvenly,
                  children: [
                    FlatButton(
                        child: Text(
                          "ลืมรหัสผ่าน",
                          style:
                              TextStyle(color: Color(0xff8a56ac), fontSize: 12),
                        ),
                        onPressed: () {
                          Navigator.pushNamed(context, ForgotPassPage.routeName,
                              arguments:
                              ForgotPassPageArguments(viewModel.memberType));
                        }),
                    FlatButton(
                        child: Text(
                          "ลงทะเบียน",
                          style:
                              TextStyle(color: viewModel.memberType == 1 ? Colors.grey : Color(0xff8a56ac), fontSize: 12),
                        ),
                        onPressed: viewModel.memberType == 1? null : () {
                          Navigator.pushNamed(context, RegisterPage.routeName,
                              arguments:
                                  RegisterPageArguments(viewModel.memberType));
                        })
                  ],
                )
              ],
            ),
          ),
        ),
      ),
    );
  }

  List<Widget> type1UserForm() {
    return [
      Padding(
        padding: const EdgeInsets.only(left: 56, right: 56),
        child: TextFormField(
          controller: viewModel.usernameController,
          decoration: InputDecoration(
              labelText: "รหัสพนักงาน",
              labelStyle: TextStyle(color: Color(0xFF767676)),
              focusedBorder: UnderlineInputBorder(
                borderSide: BorderSide(color: Color(0xFF352641)),
              ),
              enabledBorder: UnderlineInputBorder(
                borderSide: BorderSide(color: Color(0xFFDDDDDD)),
              )),
        ),
      ),
      Padding(
        padding: const EdgeInsets.only(left: 56, right: 56),
        child: TextFormField(
          obscureText: true,
          controller: viewModel.passwordController,
          decoration: InputDecoration(
              labelText: "รหัสผ่าน (Password)",
              labelStyle: TextStyle(color: Color(0xFF767676)),
              focusedBorder: UnderlineInputBorder(
                borderSide: BorderSide(color: Color(0xFF352641)),
              ),
              enabledBorder: UnderlineInputBorder(
                borderSide: BorderSide(color: Color(0xFFDDDDDD)),
              ),
          ),
        ),
      ),
      Padding(
        padding: const EdgeInsets.only(left: 40, right: 56),
        child: Row(
          mainAxisAlignment: MainAxisAlignment.spaceBetween,
          children: [
            Row(
              children: [
                Checkbox(
                  value: viewModel.remember,
                  onChanged: (bool newValue) {
                    setState(() {
                      viewModel.remember = newValue;
                    });
                  },
                ),
                Text("จดจำรหัสผ่าน",
                    style: TextStyle(fontSize: 12, color: Color(0xFF767676))),
              ],
            ),
            Text("**รหัสผ่านของระบบ IDM",
                style: TextStyle(fontSize: 12, color: Color(0xFF767676)))
          ],
        ),
      ),
    ];
  }

  List<Widget> type2Or3UserForm() {
    return [
      Padding(
        padding: const EdgeInsets.only(left: 56, right: 56),
        child: TextFormField(
          controller: viewModel.usernameController,
          decoration: InputDecoration(
              labelText: "เลขที่สมาชิก/เลขประจำตัวประชาชน",
              labelStyle: TextStyle(color: Color(0xFF767676)),
              focusedBorder: UnderlineInputBorder(
                borderSide: BorderSide(color: Color(0xFF352641)),
              ),
              enabledBorder: UnderlineInputBorder(
                borderSide: BorderSide(color: Color(0xFFDDDDDD)),
              )),
        ),
      ),
      Padding(
        padding: const EdgeInsets.only(left: 56, right: 56),
        child: TextFormField(
          controller: viewModel.passwordController,
          obscureText: true,
          inputFormatters: [LengthLimitingTextInputFormatter(8),],
          decoration: InputDecoration(
              labelText: "รหัสผ่าน (Password)",
              labelStyle: TextStyle(color: Color(0xFF767676)),
              focusedBorder: UnderlineInputBorder(
                borderSide: BorderSide(color: Color(0xFF352641)),
              ),
              enabledBorder: UnderlineInputBorder(
                borderSide: BorderSide(color: Color(0xFFDDDDDD)),
              )),
        ),
      ),
      Padding(
        padding: const EdgeInsets.only(left: 40, right: 56),
        child: Row(
          mainAxisAlignment: MainAxisAlignment.spaceBetween,
          children: [
            Row(
              children: [
                Checkbox(
                  value: viewModel.remember,
                  onChanged: (bool newValue) {
                    setState(() {
                      viewModel.remember = newValue;
                    });
                  },
                ),
                Text("จดจำรหัสผ่าน",
                    style: TextStyle(fontSize: 12, color: Color(0xFF767676))),
              ],
            )
          ],
        ),
      ),
    ];
  }

  void openMainPage() {
    setState(() {
      viewModel.isLoading = false;
    });
    Navigator.pushNamedAndRemoveUntil(
        context, NewsPage.routeName, (Route<dynamic> route) => false);
  }

  verifyUser(BuildContext context, User user, LoginPageViewModel viewModel) {
    if (user.isApplyPolicy != '1') {
      openMainPage();
    } else {
      viewModel.getPolicy().then((value) => {
            showDialog(
                context: context,
                builder: (BuildContext context) {
                  return AlertDialog(
                    title: Text("Policy"),
                    content: SingleChildScrollView(child: Text(value)),
                    actions: [
                      FlatButton(
                        child: Text("ไม่ยอมรับ"),
                        onPressed: () {
                          viewModel.setNotAgreeToPolicy();
                          Navigator.of(context).pop();
                        },
                      ),
                      FlatButton(
                        child: Text("ยอมรับ"),
                        onPressed: () {
                          print('onPressed()');
                          viewModel.setAgreeToPolicy().then(
                              (value) => {
                                    setState(() {
                                      print('value = $value');
                                      if (value) {
                                        Navigator.of(context).pop();
                                        openMainPage();
                                      }
                                    })
                                  }, onError: (e) {
                            print(e);
                          });
                        },
                      )
                    ],
                  );
                })
          });
    }
  }
}
