import 'package:flutter/material.dart';
import 'package:ndialog/ndialog.dart';
import 'package:pea_cremation/pages/aboutus/aboutus_page.dart';
import 'package:pea_cremation/pages/contact/contact_page.dart';
import 'package:pea_cremation/pages/dead/dead_page.dart';
import 'package:pea_cremation/pages/download/form_download_page.dart';
import 'package:pea_cremation/pages/news/news_page.dart';
import 'package:pea_cremation/pages/payment/payment_page.dart';
import 'package:pea_cremation/pages/personal/personal_page.dart';
import 'package:pea_cremation/domain/model/gainer.dart';
import 'package:pea_cremation/domain/model/resposibility_member.dart';
import 'package:pea_cremation/shares/app_drawer.dart';
import 'package:pea_cremation/domain/model/user.dart';
import 'package:pea_cremation/pages/menu/menu_viewmodel.dart';
import 'package:provider/provider.dart';

class MenuPage extends StatefulWidget {
  static const routeName = '/menuPage';

  @override
  _MenuPageState createState() => _MenuPageState();
}

class _MenuPageState extends State<MenuPage> {
  MenuViewModel viewModel;

  @override
  Widget build(BuildContext context) {
    viewModel = Provider.of<MenuViewModel>(context);
    viewModel.loadUserData();

    return Scaffold(
        backgroundColor: Color(0xFF7F3F97),
        body: FutureBuilder<User>(
            future: viewModel.user,
            builder: (context, snapshot) {
              if (!snapshot.hasData) {
                return Container(
                  color: Colors.white,
                );
              } else {
                return SingleChildScrollView(
                  child: Padding(
                    padding: const EdgeInsets.only(top:56.0),
                    child: Column(
                        crossAxisAlignment: CrossAxisAlignment.center,
                        children: [
                          Container(
                            color: Colors.white,
                            child: getHeaderView(context, snapshot.data),
                          ),
                          Container(
                            color: Color(0xFF453654),
                            child: GridView.count(
                              primary: false,
                              padding: const EdgeInsets.all(20),
                              crossAxisSpacing: 10,
                              mainAxisSpacing: 10,
                              crossAxisCount: 2,
                              shrinkWrap: true,
                              children: <Widget>[
                                menuItem(Icons.home, "ข่าวประชาสัมพันธ์", NewsPage.routeName),
                                menuItem(Icons.person, "ข้อมูลส่วนตัว", PersonalPage.routeName),
                                menuItem(Icons.people, "รายชื่อสมาชิก\nที่ถึงแก่กรรม", DeadPage.routeName),
                                menuItem(Icons.calendar_today, "ข้อมูลการชำระเงิน", PaymentPage.routeName),
                                menuItem(Icons.print, "ดาวโหลดแบบฟอร์ม", FormDownloadPage.routeName),
                                menuItem(Icons.location_on, "ติดต่อเรา", ContactPage.routeName),
                                menuItem(Icons.info_outline, "เกี่ยวกับเรา", AboutUsPage.routeName),
                              ],
                            ),
                          )
                        ]),
                  ),
                );
              }
            }));
  }

  Widget getHeaderView(BuildContext context, User user) {
    // imageSelectorGallery() async {
    //   viewModel.imageFile = await ImagePicker().getImage(
    //     source: ImageSource.gallery,
    //   );
    //   await viewModel.uploadPhoto();
    //   Navigator.pop(context);
    //   Navigator.pushNamed(context, PersonalPage.routeName + "_");
    // }
    //
    // imageSelectorCamera() async {
    //   viewModel.imageFile = await ImagePicker().getImage(
    //     source: ImageSource.camera,
    //   );
    //   await viewModel.uploadPhoto();
    //
    //   Navigator.pop(context);
    //   Navigator.pushNamed(context, PersonalPage.routeName + "_");
    // }

    return Column(
      children: [
        SizedBox(
            width: 128,
            child: Container(
                margin: const EdgeInsets.only(top: 48),
                child: user.coverImagePath == null
                    ? Container(
                        width: 128.0,
                        height: 128.0,
                        decoration: new BoxDecoration(
                            shape: BoxShape.circle,
                            image: new DecorationImage(
                                fit: BoxFit.fill,
                                image: AssetImage('assets/user@2x.png'))))
                    : Container(
                        width: 128.0,
                        height: 128.0,
                        decoration: new BoxDecoration(
                            shape: BoxShape.circle,
                            image: new DecorationImage(
                                fit: BoxFit.fill,
                                image: NetworkImage(user.coverImagePath)))))),
        Padding(
          padding: const EdgeInsets.only(bottom: 8.0),
          child: Text("${user?.firstName} ${user?.lastName}",
              style: TextStyle(
                  fontSize: 22,
                  fontWeight: FontWeight.bold,
                  color: Color(0XFF241332))),
        ),
        Text(getMemberCodeText(user),
            style: TextStyle(fontSize: 12, color: Color(0XFF78849E))),
        ButtonBar(
          alignment: MainAxisAlignment.spaceEvenly,
          children: [
            FlatButton(
                child: Text(
                  "รายชื่อผู้รับประโยชน์",
                  style: TextStyle(
                      color: Color(0xff8a56ac),
                      fontSize: 12,
                      fontWeight: FontWeight.w700),
                ),
                onPressed: () async {
                  await DialogBackground(
                      color: Colors.white10,
                      blur: 10,
                      dialog: NDialog(
                          content: FutureBuilder<List<Gainer>>(
                        future: viewModel.getGainerList(),
                        builder: (context, snapshot) {
                          if (snapshot.hasData) {
                            return getGainerDialogContent(snapshot.data);
                          } else {
                            return LinearProgressIndicator();
                          }
                        },
                      ))).show(context);
                }),
            FlatButton(
                child: Text(
                  "รายชื่อสมาชิกที่รับผิดชอบ",
                  style: TextStyle(
                      color: Color(0xff8a56ac),
                      fontSize: 12,
                      fontWeight: FontWeight.w700),
                ),
                onPressed: () async {
                  await DialogBackground(
                      color: Colors.white10,
                      blur: 10,
                      dialog: NDialog(
                          content: FutureBuilder<List<ResposibilityMember>>(
                        future: viewModel.getResponsibleList(),
                        builder: (context, snapshot) {
                          if (snapshot.hasData) {
                            return getResponsibleDialogContent(snapshot.data);
                          } else {
                            return LinearProgressIndicator();
                          }
                        },
                      ))).show(context);
                }),
          ],
        ),
      ],
    );
  }

  Widget getGainerDialogContent(List<Gainer> data) {
    List<Widget> contents = [
      Padding(
        padding: const EdgeInsets.fromLTRB(8.0, 16, 8, 0),
        child: Text('รายชื่อผู้รับประโยชน์',
            style: TextStyle(fontSize: 20, color: Colors.black)),
      ),
    ];
    if (data.length > 0 && data[0].name != null) {
      contents.add(Padding(
        padding: const EdgeInsets.all(8.0),
        child: Text('จำนวน ${data.length} ท่าน ดังนี้',
            style: TextStyle(fontSize: 16, color: Colors.black45)),
      ));
      for (int i = 0; i < data.length; i++) {
        contents.add(Padding(
          padding: const EdgeInsets.all(8.0),
          child: Text('${i + 1}. ${data[i].name} ${data[i].surname}',
              style: TextStyle(fontSize: 16, color: Colors.black45)),
        ));
      }
    } else {
      contents.add(Padding(
        padding: const EdgeInsets.all(8.0),
        child: Text('ไม่พบรายชื่อผู้รับประโยชน์',
            style: TextStyle(fontSize: 16, color: Colors.black45)),
      ));
    }

    contents.add(Padding(
      padding: const EdgeInsets.only(top: 24.0),
      child: Container(
          alignment: Alignment.bottomRight,
          child: RaisedButton(
            onPressed: () {
              Navigator.pop(context);
            },
            child: const Text('รับทราบ', style: TextStyle(fontSize: 14)),
            color: Color(0xff9599B3),
            textColor: Colors.white,
            shape: RoundedRectangleBorder(
                borderRadius: BorderRadius.circular(32.0)),
          )),
    ));

    return Wrap(children: [
      Column(crossAxisAlignment: CrossAxisAlignment.start, children: contents),
    ]);
  }

  Widget getResponsibleDialogContent(List<ResposibilityMember> data) {
    List<Widget> contents = [
      Padding(
        padding: const EdgeInsets.fromLTRB(8.0, 16, 8, 0),
        child: Text('รายชื่อสมาชิกที่รับผิดชอบ',
            style: TextStyle(fontSize: 20, color: Colors.black)),
      ),
    ];
    if (data.length > 0 && data[0].name != null) {
      contents.add(Padding(
        padding: const EdgeInsets.all(8.0),
        child: Text('จำนวน ${data.length} ท่าน ดังนี้',
            style: TextStyle(fontSize: 16, color: Colors.black45)),
      ));
      for (int i = 0; i < data.length; i++) {
        contents.add(Padding(
          padding: const EdgeInsets.all(8.0),
          child: Text('${i + 1}. ${data[i].name} ${data[i].surname}',
              style: TextStyle(fontSize: 16, color: Colors.black45)),
        ));
      }
    } else {
      contents.add(Padding(
        padding: const EdgeInsets.all(8.0),
        child: Text('ไม่พบรายชื่อสมาชิกที่รับผิดชอบ',
            style: TextStyle(fontSize: 16, color: Colors.black45)),
      ));
    }

    contents.add(Padding(
      padding: const EdgeInsets.only(top: 24.0),
      child: Container(
          alignment: Alignment.bottomRight,
          child: RaisedButton(
            onPressed: () {
              Navigator.pop(context);
            },
            child: const Text('รับทราบ', style: TextStyle(fontSize: 14)),
            color: Color(0xff9599B3),
            textColor: Colors.white,
            shape: RoundedRectangleBorder(
                borderRadius: BorderRadius.circular(32.0)),
          )),
    ));

    return Wrap(children: [
      Column(crossAxisAlignment: CrossAxisAlignment.start, children: contents),
    ]);
  }

  Widget menuItem(IconData icon, String title, String routeName) {
    return InkWell(
      onTap: () {
        print('click on $title ($routeName)');
        Navigator.pushNamed(context, routeName);
      },
      child: Container(
        padding: const EdgeInsets.all(8),
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.center,
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            Icon(icon, size: 64.0, color: Color(0xff8A56AC),),
            Text(title, style: TextStyle(fontSize: 18), textAlign: TextAlign.center,)
          ],
        ),
        decoration: new BoxDecoration(
            color: Colors.white,
            borderRadius: new BorderRadius.all(Radius.circular(32.0)),
            boxShadow: [
              BoxShadow(
                color: Colors.black.withOpacity(0.3),
                spreadRadius: 2,
                blurRadius: 2,
                offset: Offset(0, 3), // changes position of shadow
              ),
            ]
        ),
        // color: Colors.white,
      ),
    );
  }
}
