import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:intl/intl.dart';
import 'package:loading_overlay/loading_overlay.dart';
import 'package:pea_cremation/domain/model/invoice_info.dart';
import 'package:pea_cremation/domain/model/user.dart';
import 'package:pea_cremation/pages/invoice/invoice/invoice_view_page.dart';
import 'package:pea_cremation/pages/invoice/invoice/invoice_view_page_arguments.dart';
import 'package:pea_cremation/pages/invoice/invoice_download_page.dart';
import 'package:pea_cremation/pages/payment/barcode_view_arguments.dart';
import 'package:pea_cremation/pages/payment/payment_page_view_model.dart';
import 'package:pea_cremation/shares/app_drawer.dart';
import 'package:pea_cremation/shares/months.dart';
import 'package:provider/provider.dart';
import 'package:url_launcher/url_launcher.dart';

import 'barcode_view.dart';

class PaymentPage extends StatefulWidget {
  static const routeName = "/paymentPage";

  @override
  _PaymentPageState createState() => _PaymentPageState();
}

class _PaymentPageState extends State<PaymentPage> {
  PaymentPageViewModel viewModel;
  var isLoading = true;

  @override
  Widget build(BuildContext context) {
    viewModel = Provider.of<PaymentPageViewModel>(context);
    viewModel.getUser();

    return Scaffold(
      appBar: AppBar(
        elevation: 0,
        title: SizedBox(
          width: double.infinity,
          child: Padding(
            padding: const EdgeInsets.only(left: 24.0),
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                Text("ข้อมูลการชำระเงิน",
                    style: TextStyle(color: Colors.white, fontSize: 24))
              ],
            ),
          ),
        ),
        backgroundColor: Color(0xFF7F3F97),
        leading: AppDrawer.appBarLeadIcon(context),
      ),
      drawer: AppDrawer().getAppDrawer(context),
      body: SafeArea(
          child: Column(
        children: [
          Container(color: Color(0xFF7F3F97), child: getUserInfo(context)),
          Expanded(child: getPaymentForm(context))
        ],
      )),
    );
  }

  Widget getUserInfo(BuildContext context) {
    return Padding(
      padding: const EdgeInsets.only(left: 16.0),
      child: FutureBuilder<User>(
          future: viewModel.user,
          builder: (context, snapshot) {
            if (snapshot.hasData) {
              viewModel.memberType = snapshot.data.memberType;

              return Row(
                  mainAxisAlignment: MainAxisAlignment.start,
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: [
                    Padding(
                        padding: const EdgeInsets.only(bottom: 16.0),
                        child: snapshot.data.coverImagePath == null
                            ? Container(
                                width: 64.0,
                                height: 64.0,
                                decoration: new BoxDecoration(
                                    shape: BoxShape.circle,
                                    image: new DecorationImage(
                                        fit: BoxFit.fill,
                                        image:
                                            AssetImage('assets/user@2x.png'))))
                            : Container(
                                width: 64.0,
                                height: 64.0,
                                decoration: new BoxDecoration(
                                    shape: BoxShape.circle,
                                    image: new DecorationImage(
                                        fit: BoxFit.fill,
                                        image: NetworkImage(
                                            snapshot.data.coverImagePath))))),
                    Padding(
                      padding: const EdgeInsets.only(left: 16.0),
                      child: Column(
                        crossAxisAlignment: CrossAxisAlignment.start,
                        children: [
                          Text(
                              "${snapshot.data.firstName} ${snapshot.data.lastName}",
                              style: TextStyle(
                                  fontSize: 22,
                                  fontWeight: FontWeight.bold,
                                  color: Colors.white)),
                          Text(getMemberCodeText(snapshot.data),
                              style: TextStyle(
                                  fontSize: 12, color: Colors.white70))
                        ],
                      ),
                    )
                  ]);
            } else {
              return Container(color: Colors.transparent);
            }
          }),
    );
  }

  Widget getPaymentForm(BuildContext context) {
    return FutureBuilder<InvoiceInfo>(
        future: viewModel.getCurrentInvoice(),
        builder: (context, snapshot) {
          if (snapshot.hasData) {
            // isLoading = false;

            return ListView(children: [Padding(
              padding:
                  const EdgeInsets.only(left: 32.0, right: 32.0, top: 16.0),
              child: Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: [
                  Padding(
                    padding: const EdgeInsets.all(8.0),
                    child: Row(
                      mainAxisAlignment: MainAxisAlignment.spaceBetween,
                      crossAxisAlignment: CrossAxisAlignment.center,
                      children: [
                        Padding(
                          padding: const EdgeInsets.only(right: 8.0),
                          child: Container(
                            height: 16.0,
                            width: 16.0,
                            decoration: new BoxDecoration(
                              color: Color(0XFF8A56AC),
                              shape: BoxShape.circle,
                            ),
                          ),
                        ),
                        Expanded(
                          child: Text(
                            "ยอดยกมา",
                            style: TextStyle(fontSize: 16),
                          ),
                        ),
                        Text(
                            "${_numberFormat(snapshot.data.forwardAllowance)} บาท",
                            style: TextStyle(fontSize: 14))
                      ],
                    ),
                  ),
                  Padding(
                    padding: const EdgeInsets.only(bottom: 8.0),
                    child: Container(
                      height: 0.1,
                      color: Color(0XFF998FA2),
                    ),
                  ),
                  Padding(
                    padding: const EdgeInsets.all(8.0),
                    child: Row(
                      mainAxisAlignment: MainAxisAlignment.spaceBetween,
                      crossAxisAlignment: CrossAxisAlignment.center,
                      children: [
                        Padding(
                          padding: const EdgeInsets.only(right: 8.0),
                          child: Container(
                            height: 16.0,
                            width: 16.0,
                            decoration: new BoxDecoration(
                              color: Color(0XFF4F8DCB),
                              shape: BoxShape.circle,
                            ),
                          ),
                        ),
                        Expanded(
                          child: Text(
                            "ยอดเรียกเก็บประจำเดือน",
                            style: TextStyle(fontSize: 16),
                          ),
                        ),
                        Text("${_numberFormat(snapshot.data.allowance)} บาท",
                            style: TextStyle(fontSize: 14))
                      ],
                    ),
                  ),
                  Padding(
                    padding: const EdgeInsets.only(bottom: 8.0),
                    child: Container(
                      height: 0.1,
                      color: Color(0XFF998FA2),
                    ),
                  ),
                  Padding(
                    padding: const EdgeInsets.all(8.0),
                    child: Row(
                      mainAxisAlignment: MainAxisAlignment.spaceBetween,
                      crossAxisAlignment: CrossAxisAlignment.center,
                      children: [
                        Padding(
                          padding: const EdgeInsets.only(right: 8.0),
                          child: Container(
                            height: 16.0,
                            width: 16.0,
                            decoration: new BoxDecoration(
                              color: Color(0XFF5F4591),
                              shape: BoxShape.circle,
                            ),
                          ),
                        ),
                        Expanded(
                          child: Text(
                            "ยอดค้างชำระ",
                            style: TextStyle(fontSize: 16),
                          ),
                        ),
                        Text(
                            "${_numberFormat(snapshot.data.debtAllowance)} บาท",
                            style: TextStyle(fontSize: 14))
                      ],
                    ),
                  ),
                  Padding(
                    padding: const EdgeInsets.only(bottom: 8.0),
                    child: Container(
                      height: 0.1,
                      color: Color(0XFF998FA2),
                    ),
                  ),
                  Padding(
                    padding: const EdgeInsets.all(8.0),
                    child: Row(
                      mainAxisAlignment: MainAxisAlignment.spaceBetween,
                      crossAxisAlignment: CrossAxisAlignment.center,
                      children: [
                        Padding(
                          padding: const EdgeInsets.only(right: 8.0),
                          child: Container(
                            height: 16.0,
                            width: 16.0,
                            decoration: new BoxDecoration(
                              color: Color(0XFF5F4591),
                              shape: BoxShape.circle,
                            ),
                          ),
                        ),
                        Expanded(
                          child: Text(
                            "ยอดคงเหลือสุทธิ",
                            style: TextStyle(fontSize: 16),
                          ),
                        ),
                        Text("${_numberFormat(snapshot.data.total)} บาท",
                            style: TextStyle(fontSize: 14))
                      ],
                    ),
                  ),
                  Padding(
                    padding: const EdgeInsets.only(top: 16.0),
                    child: Text(
                      getTextByInvoice(snapshot.data),
                      style: TextStyle(fontSize: 17),
                    ),
                  ),
                  Padding(
                    padding: const EdgeInsets.only(top: 16.0),
                    child: Text(
                      'รายละเอียดการชำระเงิน',
                      style: TextStyle(fontSize: 17),
                    ),
                  ),
                  if ((viewModel.memberType == '3'))
                    Padding(
                      padding: const EdgeInsets.only(top: 16.0),
                      child: RichText(
                        text: TextSpan(
                            text: 'คลิกที่นี่เพื่อ ',
                            style: TextStyle(fontSize: 17, color: Colors.black),
                            children: [
                              TextSpan(
                                  text: 'Download',
                                  style: TextStyle(
                                      fontSize: 17, color: Colors.blue),
                                  recognizer: TapGestureRecognizer()
                                    ..onTap = () async {
                                      // Navigator.push(context, MaterialPageRoute(builder: (context) => BarcodeView()));
                                      Navigator.pushNamed(
                                          context, BarcodeView.routeName,
                                          arguments: BarcodeViewArguments(
                                              (await viewModel.user)
                                                  .memberCode));
                                    }),
                              TextSpan(
                                text: ' Barcode',
                                style: TextStyle(
                                    fontSize: 17, color: Colors.black),
                              ),
                            ]),
                      ),
                    ),
                  Padding(
                    padding: const EdgeInsets.only(top: 16.0),
                    child: RichText(
                      text: TextSpan(
                          text: 'คลิกที่นี่เพื่อ ',
                          style: TextStyle(fontSize: 17, color: Colors.black),
                          children: [
                            TextSpan(
                                text: 'Download',
                                style:
                                    TextStyle(fontSize: 17, color: Colors.blue),
                                recognizer: TapGestureRecognizer()
                                  ..onTap = () async {
                                    Navigator.pushNamed(
                                        context, InvoiceViewPage.routeName,
                                        arguments: InvoiceViewPageArguments(
                                            snapshot.data.month,
                                            snapshot.data.year));
                                  }),
                            TextSpan(
                              text: ' ใบแจ้งหนี้',
                              style:
                                  TextStyle(fontSize: 17, color: Colors.black),
                            ),
                          ]),
                    ),
                  ),
                  if ((viewModel.memberType == '3'))
                    Padding(
                      padding: const EdgeInsets.only(top: 16.0, bottom: 32),
                      child: RichText(
                        text: TextSpan(
                            text: 'ข้อมูลใบเสร็จรับเงินและใบแจ้งหนี้ที่ผ่านมา ',
                            style: TextStyle(fontSize: 17, color: Colors.black),
                            children: [
                              TextSpan(
                                  text: 'คลิกที่นี่',
                                  style: TextStyle(
                                      fontSize: 17, color: Colors.blue),
                                  recognizer: TapGestureRecognizer()
                                    ..onTap = () async {
                                      Navigator.pushNamedAndRemoveUntil(
                                          context,
                                          InvoiceDownloadPage.routeName,
                                          (Route<dynamic> route) => false);
                                    }),
                            ]),
                      ),
                    )
                ],
              ),
            )]);
          } else {
            return LoadingOverlay(isLoading: true,color: Colors.black54, child: Container());
          }
        });
  }

  String _numberFormat(String number) {
    try {
      var numberFormat = NumberFormat("#,##0");

      return numberFormat.format(int.parse(number).abs());
    } catch (exception) {
      return number;
    }
  }

  String getTextByInvoice(InvoiceInfo data) {
    if (data.isDeadDebtAllowance3M == "Y") {
      return '\t\t\t\t ท่านมียอดค้างชำระติดต่อกันเป็นเวลา 3 เดือน จำนวน ${_numberFormat(data.debtAllowance)} บาท กรุณาชำระภายใน วันที่ ${data.fixDate}\n\nหากท่านไม่ชำระภายในเวลาที่กำหนด จะถูกตัดสิทธิจากการเป็นสมาชิก\n\nขออภัยหากท่านได้ชำระแล้ว\n';
    } else if (int.parse(data.debtAllowance) > 0 &&
        int.parse(data.total) <= 0) {
      return '\t\t\t\t ท่านมียอดค้างชำระจำนวน ${_numberFormat(data.debtAllowance)} บาท\nกรุณาชำระภายในวันที่ ${data.fixDate} ขออภัยหากท่านได้ชำระแล้ว';
    } else {
      return '\t\t\t\t หากต้องการชำระเงินเพิ่มสามารถ ดำเนินการตามรายละเอียดการชำระเงินด้านล่างนี้';
    }

    // |099400016550101 322900  0
  }
}
