import 'dart:convert';
import 'dart:ui';

import 'package:flutter/material.dart';
import 'package:image_picker/image_picker.dart';
import 'package:ndialog/ndialog.dart';
import 'package:pea_cremation/domain/model/district.dart';
import 'package:pea_cremation/domain/model/gainer.dart';
import 'package:pea_cremation/domain/model/province.dart';
import 'package:pea_cremation/domain/model/resposibility_member.dart';
import 'package:pea_cremation/domain/model/sub_district.dart';
import 'package:pea_cremation/domain/model/user.dart';
import 'package:pea_cremation/pages/changepass/change_pass_page.dart';
import 'package:pea_cremation/pages/personal/personal_page_view_model.dart';
import 'package:pea_cremation/shares/alert_dialog.dart';
import 'package:pea_cremation/shares/app_drawer.dart';
import 'package:pea_cremation/shares/custom_date_picker_dialog.dart';
import 'package:provider/provider.dart';

class PersonalPage extends StatefulWidget {
  static const routeName = '/personalPage';

  @override
  _PersonalPageState createState() => _PersonalPageState();
}

class _PersonalPageState extends State<PersonalPage> {
  PersonalPageViewModel viewModel;
  bool _chk = false;

  @override
  Widget build(BuildContext context) {
    viewModel = Provider.of<PersonalPageViewModel>(context);
    viewModel.loadUserData();
    // viewModel.addListener(() {
    //   setState(() { });
    // });

    return Scaffold(
      backgroundColor: Color(0xff241332),
      extendBodyBehindAppBar: true,
      appBar: AppBar(
        brightness: Brightness.light,
        elevation: 0,
        leading: new IconButton(
          icon: new Icon(
            Icons.arrow_back,
            color: Colors.black,
          ),
          onPressed: () => Navigator.of(context).pop(),
        ),
        backgroundColor: Colors.transparent,
      ),
      body: FutureBuilder<User>(
          future: viewModel.user,
          builder: (context, snapshot) {
            if (!snapshot.hasData) {
              return Container(
                color: Colors.white,
              );
            } else {
              print('########## ${snapshot.data.receiveLetter}');
              viewModel.receiveLetter =
                  snapshot.data.receiveLetter == "1" ? "1" : "0";

              return SingleChildScrollView(
                child: Column(
                  crossAxisAlignment: CrossAxisAlignment.center,
                  children: [
                    Container(
                      color: Colors.white,
                      child: getHeaderView(context, snapshot.data),
                    ),
                    Container(
                      margin: const EdgeInsets.only(left: 24, right: 24),
                      child: Padding(
                        padding: const EdgeInsets.only(top: 16.0),
                        child: Card(
                          clipBehavior: Clip.antiAlias,
                          child: Padding(
                            padding: const EdgeInsets.all(16.0),
                            child: SizedBox(
                              width: double.infinity,
                              child: Column(
                                crossAxisAlignment: CrossAxisAlignment.start,
                                children: [
                                  Padding(
                                    padding: const EdgeInsets.only(top: 8.0),
                                    child: Text(
                                      "ข้อมูลส่วนบุคคล",
                                      style: TextStyle(
                                          color: Color(0xff8a56ac),
                                          fontSize: 16,
                                          fontWeight: FontWeight.w600),
                                    ),
                                  ),
                                  Padding(
                                    padding: const EdgeInsets.only(top: 8.0),
                                    child: Text(
                                      "เลขประจำตัวประชาชน ${snapshot.data.personalID}",
                                      style: TextStyle(
                                          color: Color(0xff241332),
                                          fontSize: 16,
                                          fontWeight: FontWeight.w600),
                                    ),
                                  ),
                                  Padding(
                                    padding: const EdgeInsets.only(top: 8.0),
                                    child: Text(
                                      snapshot.data.memberType == "1"
                                          ? "รหัสพนักงาน ${snapshot.data.memberCode}"
                                          : "รหัสสมาชิก ${snapshot.data.memberType}-${snapshot.data.memberCode}",
                                      style: TextStyle(
                                          color: Color(0xff241332),
                                          fontSize: 16,
                                          fontWeight: FontWeight.w600),
                                    ),
                                  ),
                                  Padding(
                                    padding: const EdgeInsets.only(top: 24.0),
                                    child: Text(
                                      "ข้อมูลติดต่อ",
                                      style: TextStyle(
                                          color: Color(0xff8a56ac),
                                          fontSize: 16,
                                          fontWeight: FontWeight.w600),
                                    ),
                                  ),
                                  Padding(
                                    padding: const EdgeInsets.only(top: 8),
                                    child: TextFormField(
                                      controller: viewModel.emailController
                                        ..text = snapshot.data.email,
                                      readOnly: snapshot.data.memberType == "1",
                                      enabled: snapshot.data.memberType != "1",
                                      decoration: InputDecoration(
                                          labelText: "Email",
                                          labelStyle: TextStyle(
                                              color: Color(0xFF767676)),
                                          focusedBorder: UnderlineInputBorder(
                                            borderSide: BorderSide(
                                                color: Color(0xFF352641)),
                                          ),
                                          enabledBorder: UnderlineInputBorder(
                                            borderSide: BorderSide(
                                                color: Color(0xFFDDDDDD)),
                                          )),
                                    ),
                                  ),
                                  Padding(
                                    padding: const EdgeInsets.only(top: 8),
                                    child: TextFormField(
                                      readOnly: snapshot.data.memberType == "1",
                                      enabled: snapshot.data.memberType != "1",
                                      controller: viewModel.mobileController
                                        ..text = snapshot.data.mobile,
                                      decoration: InputDecoration(
                                          labelText: "โทรศัพท์*",
                                          labelStyle: TextStyle(
                                              color: Color(0xFF767676)),
                                          focusedBorder: UnderlineInputBorder(
                                            borderSide: BorderSide(
                                                color: Color(0xFF352641)),
                                          ),
                                          enabledBorder: UnderlineInputBorder(
                                            borderSide: BorderSide(
                                                color: Color(0xFFDDDDDD)),
                                          )),
                                    ),
                                  ),
                                  Padding(
                                    padding: const EdgeInsets.only(top: 8.0),
                                    child: TextFormField(
                                      readOnly: true,
                                      enabled: snapshot.data.memberType != "1",
                                      controller: viewModel.birthDateController
                                        ..text = snapshot.data.displayBirthDate,
                                      onTap: () async {
                                        var picked = await showCustomDatePicker(
                                          context: context,
                                          initialDate: viewModel.getBirthDate(),
                                          firstDate: DateTime(1950),
                                          lastDate: DateTime(2025),
                                        );
                                        if (picked != null) {
                                          setState(() {
                                            viewModel.setBirthDate(picked);
                                          });
                                        }
                                      },
                                      decoration: InputDecoration(
                                          labelText: "วัน/เดือน/ปี เกิด",
                                          labelStyle: TextStyle(
                                              color: Color(0xFF767676)),
                                          focusedBorder: UnderlineInputBorder(
                                            borderSide: BorderSide(
                                                color: Color(0xFF352641)),
                                          ),
                                          enabledBorder: UnderlineInputBorder(
                                            borderSide: BorderSide(
                                                color: Color(0xFFDDDDDD)),
                                          )),
                                    ),
                                  ),
                                  // Padding(
                                  //   padding: const EdgeInsets.only(top: 16.0),
                                  //   child: CheckboxListTile(
                                  //     value: snapshot.data.receiveLetter == '1',
                                  //     title: Text(
                                  //         'ยินยอมที่จะรับใบแจ้งหนี้และใบเสร็จรับเงินผ่านทาง Application'),
                                  //     controlAffinity:
                                  //         ListTileControlAffinity.leading,
                                  //     onChanged: (value) {
                                  //       setState(() {
                                  //         snapshot.data.receiveLetter =
                                  //             value ? '1' : '0';
                                  //         viewModel.receiveLetter =
                                  //             value ? '1' : '0';
                                  //       });
                                  //     },
                                  //   ),
                                  // ),
                                  Padding(
                                    padding: const EdgeInsets.only(top: 24.0),
                                    child: Text(
                                      "ที่อยู่ปัจจุบัน",
                                      style: TextStyle(
                                          color: Color(0xff8a56ac),
                                          fontSize: 16,
                                          fontWeight: FontWeight.w600),
                                    ),
                                  ),
                                  Padding(
                                    padding: const EdgeInsets.only(top: 8),
                                    child: TextFormField(
                                      readOnly: snapshot.data.memberType == "1",
                                      enabled: snapshot.data.memberType != "1",
                                      keyboardType: TextInputType.multiline,
                                      maxLines: null,
                                      controller:
                                          viewModel.addressCodeController
                                            ..text = snapshot.data.address,
                                      decoration: InputDecoration(
                                          labelText: "เลขที่/ซอย/อาคาร",
                                          labelStyle: TextStyle(
                                              color: Color(0xFF767676)),
                                          focusedBorder: UnderlineInputBorder(
                                            borderSide: BorderSide(
                                                color: Color(0xFF352641)),
                                          ),
                                          enabledBorder: UnderlineInputBorder(
                                            borderSide: BorderSide(
                                                color: Color(0xFFDDDDDD)),
                                          )),
                                    ),
                                  ),
                                  if (snapshot.data.memberType == "1")
                                    SizedBox(
                                      width: double.infinity,
                                      child: Padding(
                                        padding: const EdgeInsets.symmetric(
                                            vertical: 6),
                                        child: Text(
                                            "สมาชิกประเภท 1 ไม่สามารถเปลี่ยนข้อมูลส่วนตัวได้",
                                            style: TextStyle(
                                                fontSize: 12,
                                                color: Colors.black54),
                                            textAlign: TextAlign.right),
                                      ),
                                    ),
                                  addressSection(context, snapshot)
                                ],
                              ),
                            ),
                          ),
                        ),
                      ),
                    ),
                    Padding(
                      padding: const EdgeInsets.fromLTRB(48, 40, 48, 8),
                      child: ButtonTheme(
                        minWidth: MediaQuery.of(context).size.width,
                        height: 48,
                        child: FlatButton(
                            child: Text(
                              "บันทึกการแก้ไข",
                              style: TextStyle(color: Colors.white),
                            ),
                            onPressed: () {
                              viewModel.updateProfileData().then(
                                  (user) => {
                                        showAlertDialog(context,
                                            'บันทึกการแก้ไขข้อมูลเรียบร้อยแล้ว')
                                      }, onError: (e) {
                                setState(() {
                                  try {
                                    viewModel.isLoading = false;
                                    showAlertDialog(
                                        context,
                                        json.decode(e
                                            .toString()
                                            .split('Exception: ')
                                            .last)["message"]);
                                  } catch (e) {
                                    showAlertDialog(context,
                                        "เกิดข้อผิดพลาด กรุณาลองอีกครั้งในภายหลัง");
                                  }
                                });
                              });
                            },
                            color: Color(0xff8a56ac),
                            shape: new RoundedRectangleBorder(
                                borderRadius: new BorderRadius.circular(24.0))),
                      ),
                    ),
                    Padding(
                      padding: const EdgeInsets.fromLTRB(48, 8, 48, 32),
                      child: ButtonTheme(
                        minWidth: MediaQuery.of(context).size.width,
                        height: 48,
                        child: FlatButton(
                            child: Text(
                              "แก้ไขรหัสผ่าน",
                              style: TextStyle(color: Colors.white),
                            ),
                            onPressed: () {
                              Navigator.pushNamed(
                                  context, ChangePassPage.routeName);
                            },
                            color: Color(0xff8a56ac),
                            shape: new RoundedRectangleBorder(
                                borderRadius: new BorderRadius.circular(24.0))),
                      ),
                    )
                  ],
                ),
              );
            }
          }),
    );
  }

  Widget getHeaderView(BuildContext context, User user) {
    imageSelectorGallery() async {
      viewModel.imageFile = await ImagePicker().getImage(
        source: ImageSource.gallery,
      );
      await viewModel.uploadPhoto();
      Navigator.pop(context);
      Navigator.pushNamed(context, PersonalPage.routeName + "_");
    }

    imageSelectorCamera() async {
      viewModel.imageFile = await ImagePicker().getImage(
        source: ImageSource.camera,
      );
      await viewModel.uploadPhoto();

      Navigator.pop(context);
      Navigator.pushNamed(context, PersonalPage.routeName + "_");
    }

    return Column(
      children: [
        SizedBox(
            width: 128,
            child: InkWell(
              onTap: () {
                showModalBottomSheet(
                    context: context,
                    builder: (BuildContext bc) {
                      return Container(
                        child: new Wrap(
                          children: <Widget>[
                            new ListTile(
                                leading: new Icon(Icons.photo_library),
                                title: new Text('Photo Gallery'),
                                onTap: () {
                                  imageSelectorGallery();
                                  Navigator.pop(context);
                                }),
                            new ListTile(
                                leading: new Icon(Icons.photo_camera),
                                title: new Text('Camera'),
                                onTap: () {
                                  imageSelectorCamera();
                                  Navigator.pop(context);
                                }),
                          ],
                        ),
                      );
                    });
              },
              child: Container(
                  margin: const EdgeInsets.only(top: 48),
                  child: user.coverImagePath == null
                      ? Container(
                          width: 128.0,
                          height: 128.0,
                          decoration: new BoxDecoration(
                              shape: BoxShape.circle,
                              image: new DecorationImage(
                                  fit: BoxFit.fill,
                                  image: AssetImage('assets/user@2x.png'))))
                      : Container(
                          width: 128.0,
                          height: 128.0,
                          decoration: new BoxDecoration(
                              shape: BoxShape.circle,
                              image: new DecorationImage(
                                  fit: BoxFit.fill,
                                  image: NetworkImage(user.coverImagePath))))),
            )),
        Padding(
          padding: const EdgeInsets.only(bottom: 8.0),
          child: Text("${user?.firstName} ${user?.lastName}",
              style: TextStyle(
                  fontSize: 22,
                  fontWeight: FontWeight.bold,
                  color: Color(0XFF241332))),
        ),
        Text(getMemberCodeText(user),
            style: TextStyle(fontSize: 12, color: Color(0XFF78849E))),
        ButtonBar(
          alignment: MainAxisAlignment.spaceEvenly,
          children: [
            FlatButton(
                child: Text(
                  "รายชื่อผู้รับประโยชน์",
                  style: TextStyle(
                      color: Color(0xff8a56ac),
                      fontSize: 12,
                      fontWeight: FontWeight.w700),
                ),
                onPressed: () async {
                  await DialogBackground(
                      color: Colors.white10,
                      blur: 10,
                      dialog: NDialog(
                          content: FutureBuilder<List<Gainer>>(
                        future: viewModel.getGainerList(),
                        builder: (context, snapshot) {
                          if (snapshot.hasData) {
                            return getGainerDialogContent(snapshot.data);
                          } else {
                            return LinearProgressIndicator();
                          }
                        },
                      ))).show(context);
                }),
            FlatButton(
                child: Text(
                  "รายชื่อสมาชิกที่รับผิดชอบ",
                  style: TextStyle(
                      color: Color(0xff8a56ac),
                      fontSize: 12,
                      fontWeight: FontWeight.w700),
                ),
                onPressed: () async {
                  await DialogBackground(
                      color: Colors.white10,
                      blur: 10,
                      dialog: NDialog(
                          content: FutureBuilder<List<ResposibilityMember>>(
                        future: viewModel.getResponsibleList(),
                        builder: (context, snapshot) {
                          if (snapshot.hasData) {
                            return getResponsibleDialogContent(snapshot.data);
                          } else {
                            return LinearProgressIndicator();
                          }
                        },
                      ))).show(context);
                }),
          ],
        ),
      ],
    );
  }

  Widget addressSection(BuildContext context, AsyncSnapshot<User> snapshot) {
    if (snapshot.data.memberType == "1") {
      return Container();
    }
    return Column(
      children: [
        FutureBuilder<List<Province>>(
            future: viewModel.getProvinceList(),
            builder: (context, snapshot) {
              if (snapshot.hasData) {
                return Padding(
                  padding: const EdgeInsets.only(top: 8),
                  child: SizedBox(
                    width: double.infinity,
                    child: DropdownButton<Province>(
                      isExpanded: true,
                      value: viewModel.getProvince(snapshot.data),
                      hint: Text("จังหวัด"),
                      onChanged: (Province newValue) {
                        setState(() {
                          viewModel.province = newValue;
                          viewModel.district = null;
                          viewModel.subDistrict = null;
                          viewModel.getAmphureListByProvince();
                        });
                      },
                      items: snapshot.data
                          .toSet()
                          .toList()
                          .map<DropdownMenuItem<Province>>((Province value) {
                        return DropdownMenuItem<Province>(
                          value: value,
                          child: Text('${value.name}'),
                        );
                      }).toList(),
                    ),
                  ),
                );
              } else {
                return Container();
              }
            }),
        FutureBuilder<List<District>>(
            future: viewModel.amphureList,
            builder: (context, snapshot) {
              return Padding(
                padding: const EdgeInsets.only(top: 8),
                child: SizedBox(
                  width: double.infinity,
                  child: DropdownButton<District>(
                    value: viewModel.getAmphure(snapshot.data),
                    isExpanded: true,
                    hint: Text("อำเภอ/เขต"),
                    onChanged: (District newValue) {
                      setState(() {
                        viewModel.district = newValue;
                        viewModel.subDistrict = null;
                        viewModel.getDistrictListByAmphure();
                      });
                    },
                    items: snapshot.data
                        ?.map<DropdownMenuItem<District>>((District value) {
                      return DropdownMenuItem<District>(
                        value: value,
                        child: Text('${value.name}'),
                      );
                    })?.toList(),
                  ),
                ),
              );
            }),
        FutureBuilder<List<SubDistrict>>(
            future: viewModel.districtList,
            builder: (context, snapshot) {
              return Padding(
                padding: const EdgeInsets.only(top: 8),
                child: SizedBox(
                  width: double.infinity,
                  child: DropdownButton<SubDistrict>(
                    value: viewModel.getDistrict(snapshot.data),
                    isExpanded: true,
                    hint: Text("ตำบล/แขวง"),
                    onChanged: (SubDistrict newValue) {
                      setState(() {
                        viewModel.subDistrict = newValue;
                        viewModel.updateZipCode(newValue.zipCode);
                      });
                    },
                    items: snapshot.data?.map<DropdownMenuItem<SubDistrict>>(
                        (SubDistrict value) {
                      return DropdownMenuItem<SubDistrict>(
                        value: value,
                        child: Text('${value.name}'),
                      );
                    })?.toList(),
                  ),
                ),
              );
            }),
        Padding(
          padding: const EdgeInsets.only(top: 8),
          child: TextFormField(
            controller: viewModel.zipCodeController
              ..text = snapshot.data.zipCode,
            decoration: InputDecoration(
                labelText: "รหัสไปรษณีย์",
                labelStyle: TextStyle(color: Color(0xFF767676)),
                focusedBorder: UnderlineInputBorder(
                  borderSide: BorderSide(color: Color(0xFF352641)),
                ),
                enabledBorder: UnderlineInputBorder(
                  borderSide: BorderSide(color: Color(0xFFDDDDDD)),
                )),
          ),
        ),
      ],
    );
  }

  Widget getGainerDialogContent(List<Gainer> data) {
    List<Widget> contents = [
      Padding(
        padding: const EdgeInsets.fromLTRB(8.0, 16, 8, 0),
        child: Text('รายชื่อผู้รับประโยชน์',
            style: TextStyle(fontSize: 20, color: Colors.black)),
      ),
    ];
    if (data.length > 0 && data[0].name != null) {
      contents.add(Padding(
        padding: const EdgeInsets.all(8.0),
        child: Text('จำนวน ${data.length} ท่าน ดังนี้',
            style: TextStyle(fontSize: 16, color: Colors.black45)),
      ));
      for (int i = 0; i < data.length; i++) {
        contents.add(Padding(
          padding: const EdgeInsets.all(8.0),
          child: Text('${i + 1}. ${data[i].name} ${data[i].surname}',
              style: TextStyle(fontSize: 16, color: Colors.black45)),
        ));
      }
    } else {
      contents.add(Padding(
        padding: const EdgeInsets.all(8.0),
        child: Text('ไม่พบรายชื่อผู้รับประโยชน์',
            style: TextStyle(fontSize: 16, color: Colors.black45)),
      ));
    }

    contents.add(Padding(
      padding: const EdgeInsets.only(top: 24.0),
      child: Container(
          alignment: Alignment.bottomRight,
          child: RaisedButton(
            onPressed: () {
              Navigator.pop(context);
            },
            child: const Text('รับทราบ', style: TextStyle(fontSize: 14)),
            color: Color(0xff9599B3),
            textColor: Colors.white,
            shape: RoundedRectangleBorder(
                borderRadius: BorderRadius.circular(32.0)),
          )),
    ));

    return Wrap(children: [
      Column(crossAxisAlignment: CrossAxisAlignment.start, children: contents),
    ]);
  }

  Widget getResponsibleDialogContent(List<ResposibilityMember> data) {
    List<Widget> contents = [
      Padding(
        padding: const EdgeInsets.fromLTRB(8.0, 16, 8, 0),
        child: Text('รายชื่อสมาชิกที่รับผิดชอบ',
            style: TextStyle(fontSize: 20, color: Colors.black)),
      ),
    ];
    if (data.length > 0 && data[0].name != null) {
      contents.add(Padding(
        padding: const EdgeInsets.all(8.0),
        child: Text('จำนวน ${data.length} ท่าน ดังนี้',
            style: TextStyle(fontSize: 16, color: Colors.black45)),
      ));
      for (int i = 0; i < data.length; i++) {
        contents.add(Padding(
          padding: const EdgeInsets.all(8.0),
          child: Text('${i + 1}. ${data[i].name} ${data[i].surname}',
              style: TextStyle(fontSize: 16, color: Colors.black45)),
        ));
      }
    } else {
      contents.add(Padding(
        padding: const EdgeInsets.all(8.0),
        child: Text('ไม่พบรายชื่อสมาชิกที่รับผิดชอบ',
            style: TextStyle(fontSize: 16, color: Colors.black45)),
      ));
    }

    contents.add(Padding(
      padding: const EdgeInsets.only(top: 24.0),
      child: Container(
          alignment: Alignment.bottomRight,
          child: RaisedButton(
            onPressed: () {
              Navigator.pop(context);
            },
            child: const Text('รับทราบ', style: TextStyle(fontSize: 14)),
            color: Color(0xff9599B3),
            textColor: Colors.white,
            shape: RoundedRectangleBorder(
                borderRadius: BorderRadius.circular(32.0)),
          )),
    ));

    return Wrap(children: [
      Column(crossAxisAlignment: CrossAxisAlignment.start, children: contents),
    ]);
  }
}
