
import 'package:async/async.dart';
import 'package:flutter/material.dart';
import 'package:image_picker/image_picker.dart';
import 'package:intl/intl.dart';
import 'package:pea_cremation/domain/model/district.dart';
import 'package:pea_cremation/domain/model/gainer.dart';
import 'package:pea_cremation/domain/model/province.dart';
import 'package:pea_cremation/domain/model/resposibility_member.dart';
import 'package:pea_cremation/domain/model/sub_district.dart';
import 'package:pea_cremation/domain/model/user.dart';
import 'package:pea_cremation/domain/service/benefit_service.dart';
import 'package:pea_cremation/domain/service/other_service.dart';
import 'package:pea_cremation/domain/service/user_service.dart';

class PersonalPageViewModel extends ChangeNotifier {
  UserService userService = UserService();
  BenefitService benefitService = BenefitService();
  OtherService otherService = OtherService();
  ImagePicker picker = ImagePicker();

  final AsyncMemoizer<User> _memoizer = AsyncMemoizer<User>();

  var step = 1;
  var isLoading = false;
  var birthDate = DateFormat('yyyy-MM-dd').format(DateTime.now());

  Future<User> user;
  Future<String> imageUrl;

  final emailController = TextEditingController();
  final birthDateController = TextEditingController();
  final mobileController = TextEditingController();
  final addressCodeController = TextEditingController();
  final zipCodeController = TextEditingController();

  Future<List<Province>> provinceList;
  Future<List<District>> amphureList;
  Future<List<SubDistrict>> districtList;

  Province province;
  District district;
  SubDistrict subDistrict;
  String receiveLetter = "1";

  var imageFile;

  Future<User> loadUserData() {
    user = _memoizer.runOnce(() {
        user = userService.getProfile();

        initialAddressData();

        return user;
    });

    return user;
  }

  Future<List<Province>> getProvinceList() {
    provinceList = otherService.getProvinceList();

    return provinceList;
  }

  Future<List<District>> getAmphureListByProvince() {
    print(province?.name);

    if (province == null) {
      amphureList = getEmptyDistrictList();
    } else {
      amphureList = otherService.getDistrictList(province.id);
    }

    return amphureList;
  }

  Future<List<SubDistrict>> getDistrictListByAmphure() {
    if (district == null) {
      districtList = getEmptySubDistrictList();
    } else {
      districtList = otherService.getSubDistrictList(district.id);
    }

    loadUserData();

    return districtList;
  }

  Province getProvince(List<Province> data) {
    if (data == null || province == null) {
      return null;
    }

    return data.firstWhere((element) => element.id == province?.id,
        orElse: null);
  }

  District getAmphure(List<District> data) {
    if (data == null || district == null) {
      return null;
    }

    return data.firstWhere((element) => element.id == district?.id,
        orElse: null);
  }

  SubDistrict getDistrict(List<SubDistrict> data) {
    if (data == null || subDistrict == null) {
      return null;
    }

    return data.firstWhere((element) => element.id == subDistrict?.id,
        orElse: null);
  }

  void initialAddressData() async {
    var user = await this.user;
    addressCodeController.text = user.address;

    province = (await otherService.getProvinceList()).firstWhere((element) => element.id == user.provinceId, orElse: null);
    district = (await otherService.getDistrictList(user.provinceId)).firstWhere((element) => element.id == user.amphureId, orElse: null);
    subDistrict = (await otherService.getSubDistrictList(user.amphureId)).firstWhere((element) => element.id == user.districtId, orElse: null);

    if(subDistrict != null) {
      user.zipCode = subDistrict.zipCode;
    }

    await getProvinceList();
    await getAmphureListByProvince();
    await getDistrictListByAmphure();

    notifyListeners();
  }

  Future<User> uploadPhoto() {
    if (imageFile is PickedFile) {
      user = userService.uploadPhoto(imageFile);
    } else {
      user = userService.uploadPhoto2(imageFile);
    }

    user.then((value) => userService.saveUser(value));

    notifyListeners();

    return user;
  }

  Future<User> updateProfileData() {
    user = userService.updateProfile(
        emailController.text,
        birthDate,
        mobileController.text,
        addressCodeController.text,
        province?.id,
        district?.id,
        subDistrict?.id,
        zipCodeController.text,
        receiveLetter
    );

    user.then((value) => userService.saveUser(value));

    return user;
  }

  void setBirthDate(DateTime pickedDate) async {
    birthDate = DateFormat('yyyy-MM-dd').format(pickedDate);

    (await user).birthDate = birthDate;
    (await user).updateDisplayBirthDate();

  }

  DateTime getBirthDate() {
    return DateFormat('yyyy-MM-dd').parse(birthDate);
  }

  updateZipCode(String zipCode) async {
    (await user).zipCode = zipCode;

    notifyListeners();
  }

  Future<List<Gainer>> getGainerList() {
    return benefitService.getGainerList();
  }

  Future<List<ResposibilityMember>> getResponsibleList() {
    return benefitService.getResposibleList();
  }

  Future<List<District>> getEmptyDistrictList() async {
    return [];
  }

  Future<List<SubDistrict>> getEmptySubDistrictList() async {
    return [];
  }
}
