import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:pea_cremation/pages/news/news_page.dart';
import 'package:pea_cremation/pages/register/register_page_arguments.dart';
import 'package:pea_cremation/pages/register/register_page_view_model.dart';
import 'package:pea_cremation/shares/alert_dialog.dart';
import 'package:provider/provider.dart';

class RegisterPage extends StatefulWidget {

  static const routeName = '/registerPage';

  @override
  _RegisterPageState createState() => _RegisterPageState();
}

class _RegisterPageState extends State<RegisterPage> {

  RegisterPageViewModel viewModel;
  RegisterPageArguments args;

  @override
  Widget build(BuildContext context) {
    viewModel = Provider.of<RegisterPageViewModel>(context);
    args = ModalRoute.of(context).settings.arguments;

    viewModel.memberType = args.memberType;

    return Scaffold(
      backgroundColor: Color(0xFFF1EFF2),
      body: Container(
        decoration: BoxDecoration(
          image: DecorationImage(
              image: AssetImage("assets/bg.png"),
              fit: BoxFit.fitWidth,
              alignment: Alignment.topCenter),
        ),
        child: SafeArea(
          child: SingleChildScrollView(
            child: Column(
              children: [
                Padding(
                  padding: const EdgeInsets.only(top: 24.0),
                  child: ButtonBar(
                    alignment: MainAxisAlignment.spaceEvenly,
                    children: [
                      FlatButton(
                          child: Text(
                            "เข้าสู่ระบบ",
                            style: TextStyle(color: Colors.white, fontSize: 14),
                          ),
                          onPressed: () {
                            Navigator.pop(context);
                          }),
                      FlatButton(
                          child: Text(
                            "ลงทะเบียน",
                            style: TextStyle(
                                color: Colors.white,
                                fontSize: 14,
                                fontWeight: FontWeight.w900),
                          ),
                          onPressed: () {

                          })
                    ],
                  ),
                ),
                Padding(
                  padding: const EdgeInsets.only(left: 16.0, right: 16.0),
                  child: Card(
                      clipBehavior: Clip.antiAlias,
                      elevation: 0,
                      shape: RoundedRectangleBorder(
                          borderRadius:
                          BorderRadius.all(Radius.circular(32.0))),
                      child: Padding(
                        padding:
                        const EdgeInsets.fromLTRB(24.0, 16.0, 24.0, 56.0),
                        child: Column(
                            children:viewModel.step == 1? getStep1Form() : getStep2Form()
                        ),
                      )),
                ),
                Padding(
                  padding: const EdgeInsets.fromLTRB(48, 40, 48, 32),
                  child: ButtonTheme(
                    minWidth: MediaQuery.of(context).size.width,
                    height: 48,
                    child: FlatButton(
                        child: Text(
                          viewModel.step == 1 ? "ถัดไป" : "ยืนยันการลงทะเบียน",
                          style: TextStyle(color: Colors.white),
                        ),
                        onPressed: () {
                          if(viewModel.step == 1) {
                            viewModel.submitCheck().then((value) => {
                              setState((){ viewModel.step = 2;})
                            },
                                onError: (e) =>
                                setState(() {
                                  try {
                                    showAlertDialog(
                                        context, e
                                        .toString()
                                        .split('Exception: ')
                                        .last);
                                  } catch (e) {
                                    print(e);
                                    showAlertDialog(context,
                                        "เกิดข้อผิดพลาด กรุณาลองอีกครั้งในภายหลัง");
                                  }
                                }));
                          } else {
                            viewModel.submitRegister().then((user) => {
                              setState(() {
                                if (user.isApplyPolicy == '1') {
                                  openMainPage();
                                } else {
                                  viewModel.getPolicy().then((value) => {
                                    showDialog(
                                        context: context,
                                        builder:
                                            (BuildContext context) {
                                          return AlertDialog(
                                            title: Text("Policy"),
                                            content:
                                            SingleChildScrollView(
                                                child: Text(value)),
                                            actions: [
                                              FlatButton(
                                                child:
                                                Text("ไม่ยอมรับ"),
                                                onPressed: () {
                                                  viewModel
                                                      .setNotAgreeToPolicy();
                                                  Navigator.of(context)
                                                      .pop();
                                                },
                                              ),
                                              FlatButton(
                                                child: Text("ยอมรับ"),
                                                onPressed: () {
                                                  print('onPressed()');
                                                  viewModel
                                                      .setAgreeToPolicy()
                                                      .then(
                                                          (value) => {
                                                        setState(
                                                                () {
                                                              if (value) {
                                                                Navigator.of(context).pop();
                                                                openMainPage();
                                                              }
                                                            })
                                                      },
                                                      onError: (e) {
                                                        print(e);
                                                      });
                                                },
                                              )
                                            ],
                                          );
                                        })
                                  });
                                }
                              })
                            },
                                onError: (e) =>
                                    setState(() {
                                      print(e);
                                      try {
                                        showAlertDialog(
                                            context, e
                                            .toString()
                                            .split('Exception: ')
                                            .last);
                                      } catch (e) {
                                        print(e);
                                        showAlertDialog(context,
                                            "เกิดข้อผิดพลาด กรุณาลองอีกครั้งในภายหลัง");
                                      }
                                    }));
                          }
                        },
                        color: Color(0xff8a56ac),
                        shape: new RoundedRectangleBorder(
                            borderRadius: new BorderRadius.circular(24.0))),
                  ),
                ),
                Text(
                  "ในกรณีที่ไม่สามารถ Reset รหัสผ่านได้\nกรุณาติดต่อ 02-590-5227\nหรือ 02-590-9489",
                  textAlign: TextAlign.center,
                )
              ],
            ),
          ),
        ),
      ),
    );
  }

  List<Widget> getStep1Form() {
    return [
      Visibility(
        visible: true,
        child: TextFormField(
          controller: viewModel.firstNameController,
          decoration: InputDecoration(
              labelText: "ชื่อ *",
              labelStyle:
              TextStyle(color: Color(0xFF767676)),
              focusedBorder: UnderlineInputBorder(
                borderSide:
                BorderSide(color: Color(0xFF352641)),
              ),
              enabledBorder: UnderlineInputBorder(
                borderSide:
                BorderSide(color: Color(0xFFDDDDDD)),
              )),
        ),
      ),
      Visibility(
        visible: true,
        child: TextFormField(
          controller: viewModel.lastNameController,
          decoration: InputDecoration(
              labelText: "นามสกุล *",
              labelStyle:
              TextStyle(color: Color(0xFF767676)),
              focusedBorder: UnderlineInputBorder(
                borderSide:
                BorderSide(color: Color(0xFF352641)),
              ),
              enabledBorder: UnderlineInputBorder(
                borderSide:
                BorderSide(color: Color(0xFFDDDDDD)),
              )),
        ),
      ),
      Visibility(
        visible: true,
        child: TextFormField(
          controller: viewModel.memberCodeController,
          decoration: InputDecoration(
              prefixText: "${viewModel.memberType}-",
              labelText: "เลขที่สมาชิก *",
              labelStyle:
              TextStyle(color: Color(0xFF767676)),
              focusedBorder: UnderlineInputBorder(
                borderSide:
                BorderSide(color: Color(0xFF352641)),
              ),
              enabledBorder: UnderlineInputBorder(
                borderSide:
                BorderSide(color: Color(0xFFDDDDDD)),
              )),
        ),
      )
    ];
  }

  List<Widget> getStep2Form() {
    return [
      Visibility(
        visible: true,
        child: TextFormField(
          controller: viewModel.idCardController,
          maxLength: 13,
          decoration: InputDecoration(
              labelText: "เลขประจำตัวประชาชน *",
              labelStyle:
              TextStyle(color: Color(0xFF767676)),
              focusedBorder: UnderlineInputBorder(
                borderSide:
                BorderSide(color: Color(0xFF352641)),
              ),
              enabledBorder: UnderlineInputBorder(
                borderSide:
                BorderSide(color: Color(0xFFDDDDDD)),
              )),
        ),
      ),
      Visibility(
        visible: true,
        child: TextFormField(
          controller: viewModel.emailController,
          decoration: InputDecoration(
              labelText:
              "Email",
              labelStyle:
              TextStyle(color: Color(0xFF767676)),
              focusedBorder: UnderlineInputBorder(
                borderSide:
                BorderSide(color: Color(0xFF352641)),
              ),
              enabledBorder: UnderlineInputBorder(
                borderSide:
                BorderSide(color: Color(0xFFDDDDDD)),
              )),
        ),
      ),
      Visibility(
        visible: true,
        child: TextFormField(
          controller: viewModel.passwordController,
          obscureText: true,
          inputFormatters: [LengthLimitingTextInputFormatter(8),],
          decoration: InputDecoration(
              labelText: "รหัสผ่าน (Password)*",
              labelStyle:
              TextStyle(color: Color(0xFF767676)),
              focusedBorder: UnderlineInputBorder(
                borderSide:
                BorderSide(color: Color(0xFF352641)),
              ),
              enabledBorder: UnderlineInputBorder(
                borderSide:
                BorderSide(color: Color(0xFFDDDDDD)),
              )),
        ),
      ),
      Visibility(
        visible: true,
        child: TextFormField(
          controller: viewModel.confirmPasswordController,
          obscureText: true,
          inputFormatters: [LengthLimitingTextInputFormatter(8),],
          decoration: InputDecoration(
              labelText:
              "ยืนยันรหัสผ่านใหม่ (Comfirm Password)",
              labelStyle:
              TextStyle(color: Color(0xFF767676)),
              focusedBorder: UnderlineInputBorder(
                borderSide:
                BorderSide(color: Color(0xFF352641)),
              ),
              enabledBorder: UnderlineInputBorder(
                borderSide:
                BorderSide(color: Color(0xFFDDDDDD)),
              )),
        ),
      )
    ];
  }

  void openMainPage() {
    setState(() {
      viewModel.isLoading = false;
    });
    Navigator.pushNamedAndRemoveUntil(
        context, NewsPage.routeName, (Route<dynamic> route) => false);
  }
}
