import 'package:flutter/material.dart';
import 'package:pea_cremation/domain/model/user.dart';
import 'package:pea_cremation/domain/service/user_service.dart';
import 'package:pea_cremation/shares/form_validator.dart';

class RegisterPageViewModel extends ChangeNotifier {

  UserService userService = UserService();

  var memberType = 1;
  var step = 1;
  var isLoading = false;

  final firstNameController = TextEditingController();
  final lastNameController = TextEditingController();
  final memberCodeController = TextEditingController();
  final idCardController = TextEditingController();
  final emailController = TextEditingController();
  final passwordController = TextEditingController();
  final confirmPasswordController = TextEditingController();


  Future<Map<String, dynamic>> submitCheck() async {
    if (invalidStep1Input()) {
      throw Exception();
    }

    Map<String, dynamic> response = await userService.preRegister(memberCodeController.text, firstNameController.text, lastNameController.text, memberType);

    if (response['code'] == 200) {
      step = 2;
    } else {
      throw Exception(response['message']);
    }

    print(step);

    return response;
  }

  Future<User> submitRegister() async {
    if (invalidInput()) {
      throw Exception();
    }

    return userService.register(memberCodeController.text, idCardController.text, emailController.text, passwordController.text, memberType);
  }

  bool invalidStep1Input() {
    if(firstNameController.text.trim() == '') {
      throw Exception('กรุณากรอกชื่อ');
    } else if(lastNameController.text.trim() == '') {
      throw Exception('กรุณากรอกนามสกุล');
    } else if(memberCodeController.text.trim() == '') {
      throw Exception('กรุณากรอกเลขที่สมาชิก');
    }

    return false;
  }

  bool invalidInput() {
    if(idCardController.text.trim() == '') {
      throw Exception('กรุณากรอกเลขประจำตัวประชาชน');
    } else if(invalidIdCard(idCardController.text)) {
      throw Exception('เลขบัตรประจำตัวประชาชนไม่ถูกต้อง');
    } else if(passwordController.text.length < 8) {
      throw Exception('กรุณากำหนดรหัสผ่าน 8 หลัก');
    } else if(passwordController.text != confirmPasswordController.text) {
      throw Exception('กรุณากรอกรหัสผ่านให้ตรงกัน');
    } else if(!validFormatPassword(passwordController.text)) {
      throw Exception('กรุณากำหนดรหัสผ่านให้ประกอบไปด้วยตัวเลขตัวอักษร และอักขระ');
    }

    return false;
  }

  bool notValidEmail(String email) {
    return !RegExp(r"^[a-zA-Z0-9.a-zA-Z0-9.!#$%&'*+-/=?^_`{|}~]+@[a-zA-Z0-9]+\.[a-zA-Z]+").hasMatch(email);
  }

  Future<String> getPolicy() {
    return userService.getPolicy();
  }

  Future<bool> setAgreeToPolicy() async {
    return userService.applyPolicy();
  }

  void setNotAgreeToPolicy() async {
    userService.clearUserData();
  }

  bool validFormatPassword(String password) {
    var hasCharacters = RegExp('[a-zA-Z]').hasMatch(password);
    var hasNumbers = RegExp('[0-9]').hasMatch(password);
    var hasSymbol = RegExp(r'[!@#$%^&*(),.?":{}|<>]').hasMatch(password);

    return hasCharacters && hasNumbers && hasSymbol;
  }
}