import 'package:flutter/material.dart';
import 'package:package_info/package_info.dart';
import 'package:pea_cremation/domain/model/user.dart';
import 'package:pea_cremation/domain/service/base_service.dart';
import 'package:pea_cremation/domain/service/user_service.dart';
import 'package:pea_cremation/pages/aboutus/aboutus_page.dart';
import 'package:pea_cremation/pages/contact/contact_page.dart';
import 'package:pea_cremation/pages/dead/dead_page.dart';
import 'package:pea_cremation/pages/download/form_download_page.dart';
import 'package:pea_cremation/pages/invoice/invoice_download_page.dart';
import 'package:pea_cremation/pages/login/login_page.dart';
import 'package:pea_cremation/pages/news/news_page.dart';
import 'package:pea_cremation/pages/payment/payment_page.dart';
import 'package:pea_cremation/pages/personal/personal_page.dart';

class AppDrawer {
  static bool buildForModule = false;

  static Widget appBarLeadIcon(BuildContext context,
      {Color color = Colors.white}) {
    if (!buildForModule) {
      return null;
    }
    return new IconButton(
      icon: new Icon(
        Icons.arrow_back,
        color: color,
      ),
      onPressed: () => Navigator.of(context).pop(),
    );
  }

  Widget getAppDrawer(BuildContext context) {
    if (buildForModule) {
      return null;
    }
    BaseService baseService = BaseService();
    Future<User> user = baseService.loadUser();

    return Drawer(
        child: ListView(
      // Important: Remove any padding from the ListView.
      padding: EdgeInsets.zero,
      children: <Widget>[
        SizedBox(
            height: 232,
            child: FutureBuilder<User>(
                future: user,
                builder: (context, snapshot) {
                  if (snapshot.hasData) {
                    return getDrawerHeader(context, snapshot.data);
                  } else {
                    return getEmptyDrawerHeader(context);
                  }
                })),
        ListTile(
          title: Row(children: [
            Padding(
              padding: const EdgeInsets.all(8.0),
              child: Icon(Icons.home, color: Color(0XFFA7A0AD), size: 24.0),
            ),
            Text('ข่าวประชาสัมพันธ์')
          ]),
          onTap: () {
            Navigator.pushNamedAndRemoveUntil(
                context, NewsPage.routeName, (Route<dynamic> route) => false);
          },
        ),
        ListTile(
          title: Row(children: [
            Padding(
              padding: const EdgeInsets.all(8.0),
              child: Icon(Icons.person, color: Color(0XFFA7A0AD), size: 24.0),
            ),
            Text('ข้อมูลส่วนตัว')
          ]),
          onTap: () {
            Navigator.pop(context);
            Navigator.pushNamed(context, PersonalPage.routeName);
          },
        ),
        ListTile(
          title: Row(children: [
            Padding(
              padding: const EdgeInsets.all(8.0),
              child: Icon(Icons.people, color: Color(0XFFA7A0AD), size: 24.0),
            ),
            Text('รายชื่อสมาชิกที่ถึงแก่กรรม')
          ]),
          onTap: () {
            Navigator.pushNamedAndRemoveUntil(
                context, DeadPage.routeName, (Route<dynamic> route) => false);
          },
        ),
        ListTile(
          title: Row(children: [
            Padding(
              padding: const EdgeInsets.all(8.0),
              child: Icon(Icons.calendar_today,
                  color: Color(0XFFA7A0AD), size: 24.0),
            ),
            Text('ข้อมูลการชำระเงิน')
          ]),
          onTap: () {
            Navigator.pushNamedAndRemoveUntil(context, PaymentPage.routeName,
                (Route<dynamic> route) => false);
            // Navigator.pushNamedAndRemoveUntil(
            //     context, InvoiceDownloadPage.routeName, (Route<dynamic> route) => false);
          },
        ),
        ListTile(
          title: Row(children: [
            Padding(
              padding: const EdgeInsets.all(8.0),
              child: Icon(Icons.print, color: Color(0XFFA7A0AD), size: 24.0),
            ),
            Text('ดาวโหลดแบบฟอร์ม')
          ]),
          onTap: () {
            Navigator.pushNamedAndRemoveUntil(context,
                FormDownloadPage.routeName, (Route<dynamic> route) => false);
          },
        ),
        ListTile(
          title: Row(children: [
            Padding(
              padding: const EdgeInsets.all(8.0),
              child:
                  Icon(Icons.location_on, color: Color(0XFFA7A0AD), size: 24.0),
            ),
            Text('ติดต่อเรา')
          ]),
          onTap: () {
            Navigator.pushAndRemoveUntil(
                context,
                MaterialPageRoute(builder: (context) => ContactPage()),
                ModalRoute.withName("/ContactPage"));
          },
        ),
        ListTile(
          title: Row(children: [
            Padding(
              padding: const EdgeInsets.all(8.0),
              child: Icon(Icons.info_outline,
                  color: Color(0XFFA7A0AD), size: 24.0),
            ),
            Text('เกี่ยวกับเรา')
          ]),
          onTap: () {
            Navigator.pushAndRemoveUntil(
                context,
                MaterialPageRoute(builder: (context) => AboutUsPage()),
                ModalRoute.withName(AboutUsPage.routeName));
          },
        ),
        ListTile(
          title: Row(children: [
            Padding(
              padding: const EdgeInsets.all(8.0),
              child:
                  Icon(Icons.exit_to_app, color: Color(0XFFA7A0AD), size: 24.0),
            ),
            Text('ออกจากระบบ')
          ]),
          onTap: () {
            showDialog(
              context: context,
              builder: (BuildContext context) {
                return AlertDialog(
                  title: Text('ออกจากระบบ'),
                  content: const Text('คุณต้องการออกจากระบบ ?'),
                  actions: [
                    FlatButton(
                      child: Text('ยกเลิก'),
                      onPressed: () {
                        Navigator.of(context).pop();
                      },
                    ),
                    FlatButton(
                      child: Text('ตกลง'),
                      onPressed: () {
                        UserService()
                            .logout()
                            .then((value) => {
                                  Navigator.pushNamedAndRemoveUntil(context,
                                      LoginPage.routeName, (route) => false)
                                })
                            .catchError(() {
                          Navigator.pushNamedAndRemoveUntil(
                              context, LoginPage.routeName, (route) => false);
                        });
                        // Navigator.of(context).pop();
                      },
                    ),
                  ],
                );
              },
            );
          },
        ),
        ListTile(
          title: FutureBuilder<PackageInfo>(
            future: PackageInfo.fromPlatform(),
            builder: (context, snapshot) {
              if (snapshot.hasData) {
                return Text(
                    "version: ${snapshot.data.version} (${snapshot.data.buildNumber})");
              } else {
                return Text("version: ...");
              }
            },
          ),
        )
      ],
    )
// Populate the Drawer in the next step.
        );
  }

  Widget getDrawerHeader(BuildContext context, User user) {
    return DrawerHeader(
      padding: const EdgeInsets.only(left: 32, top: 32),
      child: Column(
          mainAxisAlignment: MainAxisAlignment.start,
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            Padding(
                padding: const EdgeInsets.only(bottom: 16.0),
                child: user.coverImagePath == null
                    ? Container(
                        width: 64.0,
                        height: 64.0,
                        decoration: new BoxDecoration(
                            shape: BoxShape.circle,
                            image: new DecorationImage(
                                fit: BoxFit.fill,
                                image: AssetImage('assets/user@2x.png'))))
                    : Container(
                        width: 64.0,
                        height: 64.0,
                        decoration: new BoxDecoration(
                            shape: BoxShape.circle,
                            image: new DecorationImage(
                                fit: BoxFit.fill,
                                image: NetworkImage(user.coverImagePath))))),
            Padding(
              padding: const EdgeInsets.only(bottom: 8.0),
              child: Text("${user.firstName} ${user.lastName}",
                  style: TextStyle(
                      fontSize: 22,
                      fontWeight: FontWeight.bold,
                      color: Colors.white)),
            ),
            Text(getMemberCodeText(user),
                style: TextStyle(fontSize: 12, color: Colors.white70))
          ]),
      decoration: BoxDecoration(
          image: DecorationImage(
        image: AssetImage('assets/top.png'),
        fit: BoxFit.cover,
      )),
    );
  }

  Widget getEmptyDrawerHeader(BuildContext context) {
    return DrawerHeader(
      padding: const EdgeInsets.only(left: 32, top: 32),
      child: Column(
          mainAxisAlignment: MainAxisAlignment.start,
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            Padding(
                padding: const EdgeInsets.only(bottom: 16.0),
                child: Container(
                    width: 64.0,
                    height: 64.0,
                    decoration: new BoxDecoration(
                        shape: BoxShape.circle,
                        image: new DecorationImage(
                            fit: BoxFit.fill,
                            image: AssetImage('assets/user@2x.png'))))),
            Padding(
              padding: const EdgeInsets.only(bottom: 8.0),
              child: Text("",
                  style: TextStyle(
                      fontSize: 22,
                      fontWeight: FontWeight.bold,
                      color: Colors.white)),
            ),
            Text("", style: TextStyle(fontSize: 12, color: Colors.white70))
          ]),
      decoration: BoxDecoration(
          image: DecorationImage(
        image: AssetImage('assets/top.png'),
        fit: BoxFit.cover,
      )),
    );
  }
}

String getMemberCodeText(User user) {
  return "เลขที่สมาชิก ${user.memberType}- ${user.memberType == '1' ? user.serial : user.memberCode}";
}
